@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model;
-$Config::Model::VERSION = '2.082';
+$Config::Model::VERSION = '2.097';
 use strict ;
 use warnings;
 use 5.10.1;
@@ -25,12 +25,22 @@ use Config::Model::Instance;
 use Hash::Merge 0.12 qw/merge/;
 use File::Path qw/make_path/;
 
+use Cwd;
+use Config::Model::Lister;
+
+use parent qw/Exporter/;
+our @EXPORT_OK = qw/cme/;
+
 # this class holds the version number of the package
 use vars qw(@status @level %default_property);
 
 my $legacy_logger = get_logger("Model::Legacy") ;
 my $logger = get_logger("Model") ;
 
+# used to keep one Config::Model object to simplify programs based on
+# cme function
+my $model_storage;
+
 %default_property = (
     status      => 'standard',
     level       => 'normal',
@@ -43,7 +53,17 @@ enum LegacyTreament => qw/die warn ignore/;
 has skip_include => ( isa => 'Bool',           is => 'ro', default => 0 );
 has model_dir    => ( isa => 'Str',            is => 'ro', default => 'Config/Model/models' );
 has legacy       => ( isa => 'LegacyTreament', is => 'ro', default => 'warn' );
-has instances => ( isa => 'HashRef[Config::Model::Instance]', is => 'ro', default => sub { {} } );
+has instances => (
+    isa => 'HashRef[Config::Model::Instance]',
+    is => 'ro',
+    default => sub { {} },
+    traits => ['Hash'],
+    handles => {
+        store_instance => 'set',
+        get_instance   => 'get',
+        has_instance   => 'defined',
+    },
+);
 
 # Config::Model stores 3 versions of each model
 
@@ -156,47 +176,84 @@ sub BUILD {
 
 sub show_legacy_issue {
     my $self     = shift;
-    my $behavior = $self->legacy;
+    my $ref      = shift;
+    my $behavior = shift || $self->legacy;
 
+    my @msg = ref $ref ? @$ref : $ref;
     if ( $behavior eq 'die' ) {
-        die @_, "\n";
+        die @msg, "\n";
     }
     elsif ( $behavior eq 'warn' ) {
-        warn @_, "\n";
+        warn @msg, "\n";
+    } elsif ( $behavior eq 'note' ) {
+        say @msg;
     }
 }
 
+sub _tweak_instance_args {
+    my ($args) = @_  ;
+
+    my $application = $args->{application} ;
+    my $cat = '';
+    if (defined $application) {
+        my ( $categories, $appli_info, $appli_map ) = Config::Model::Lister::available_models;
+        $args->{root_class_name} ||= $appli_map->{$application} ;
+        $cat = $appli_info->{_category} //  ''; # may be empty in tests
+        # config_dir may be specified in application file
+        $args->{config_dir} //= $appli_info->{$application}{config_dir};
+    }
+
+    my $app_name = $application;
+    if ($cat eq 'application') {
+        # store dir in name to distinguish different runs of the same
+        # app in different directories.
+        $application .= " in " . cwd;
+    }
+    $args->{name}
+        =  delete $args->{instance_name} # backward compat with test
+        || delete $args->{name}          # preferred parameter
+        || $app_name                     # fallback in most cases
+        || 'default';                    # fallback mostly in tests
+}
+
+sub cme {
+    my %args = @_ == 1 ? ( application => $_[0]) : @_ ;
+
+    my $cat =_tweak_instance_args(\%args);
+
+    my $m_args = delete $args{model_args} // {} ; # used for tests
+    # model_storage is used to keep Config::Model object alive
+    $model_storage //= Config::Model->new(%$m_args);
+
+    my $app = $args{application};
+    return $model_storage->instance(@_);
+}
+
 sub instance {
     my $self = shift;
-    my %args = @_;
+    my %args = @_ == 1 ? ( application => $_[0]) : @_ ;
 
-    my $instance_name =
-           delete $args{instance_name}
-        || delete $args{name}
-        || 'default';
+    # also creates a default name
+    _tweak_instance_args(\%args);
 
-    # could add more syntactic suger with 'hash' trait
-    # see Moose::Meta::Attribute::Native
-    if ( defined $self->instances->{$instance_name} ) {
-        return $self->instances->{$instance_name};
+    if ( $args{name} and $self->has_instance($args{name}) ) {
+        return $self->get_instance($args{name});
     }
 
-    my $root_class_name = delete $args{root_class_name}
-        or croak "Model: can't create instance without root_class_name ";
+    croak "Model: can't create instance without application or root_class_name "
+        unless $args{root_class_name};
 
     if ( defined $args{model_file} ) {
         my $file = delete $args{model_file};
-        $self->load( $root_class_name, $file );
+        $self->load( $args{root_class_name}, $file );
     }
 
     my $i = Config::Model::Instance->new(
         config_model    => $self,
-        root_class_name => $root_class_name,
-        name            => $instance_name,
         %args    # for optional parameters like *directory
     );
 
-    $self->instances->{$instance_name} = $i;
+    $self->store_instance($args{name}, $i);
     return $i;
 }
 
@@ -361,8 +418,8 @@ sub normalize_class_parameters {
     }
 
     if ( defined $normalized_model->{inherit_after} ) {
-        $self->show_legacy_issue( "Model $config_class_name: inherit_after is deprecated ",
-            "in favor of include_after" );
+        $self->show_legacy_issue([ "Model $config_class_name: inherit_after is deprecated ",
+            "in favor of include_after" ]);
         $normalized_model->{include_after} = delete $normalized_model->{inherit_after};
     }
     if ( defined $normalized_model->{inherit} ) {
@@ -492,6 +549,8 @@ sub translate_legacy_info {
     my $elt_name          = shift;
     my $info              = shift;
 
+    $self->translate_warped_node_info( $config_class_name, $elt_name, 'warped_node', $info );
+
     #translate legacy warp information
     if ( defined $info->{warp} ) {
         $self->translate_warp_info( $config_class_name, $elt_name, $info->{type}, $info->{warp} );
@@ -499,6 +558,12 @@ sub translate_legacy_info {
 
     $self->translate_cargo_info( $config_class_name, $elt_name, $info );
 
+    if (   defined $info->{cargo}
+        && defined $info->{cargo}{type}
+        && $info->{cargo}{type} eq 'warped_node' ) {
+        $self->translate_warped_node_info( $config_class_name, $elt_name, 'warped_node', $info->{cargo} );
+    }
+
     if (    defined $info->{cargo}
         and defined $info->{cargo}{warp} ) {
         $self->translate_warp_info(
@@ -507,16 +572,6 @@ sub translate_legacy_info {
             $info->{cargo}{warp} );
     }
 
-    if (   defined $info->{cargo}
-        && defined $info->{cargo}{type}
-        && $info->{cargo}{type} eq 'warped_node' ) {
-        $self->translate_warp_info( $config_class_name, $elt_name, 'warped_node', $info->{cargo} );
-    }
-
-    if ( defined $info->{type} && $info->{type} eq 'warped_node' ) {
-        $self->translate_warp_info( $config_class_name, $elt_name, 'warped_node', $info );
-    }
-
     # compute cannot be warped
     if ( defined $info->{compute} ) {
         $self->translate_compute_info( $config_class_name, $elt_name, $info, 'compute' );
@@ -598,11 +653,11 @@ sub translate_cargo_info {
 
     if ( defined $info->{config_class_name} ) {
         $cargo{config_class_name} = delete $info->{config_class_name};
-        $self->show_legacy_issue(
+        $self->show_legacy_issue([
             "$config_class_name->$elt_name: parameter config_class_name is ",
             "deprecated. This one must be specified within cargo. ",
             "Ie. cargo=>{config_class_name => 'FooBar'}"
-        );
+        ]);
     }
 
     $info->{cargo} = \%cargo;
@@ -611,27 +666,25 @@ sub translate_cargo_info {
     ) if $legacy_logger->is_debug;
 }
 
+# TODO: set to die In September 2016
 sub translate_id_names {
     my $self              = shift;
     my $config_class_name = shift;
     my $elt_name          = shift;
     my $info              = shift;
-    $self->translate_name( $config_class_name, $elt_name, $info, 'allow',      'allow_keys' );
-    $self->translate_name( $config_class_name, $elt_name, $info, 'allow_from', 'allow_keys_from' );
-    $self->translate_name( $config_class_name, $elt_name, $info, 'follow',     'follow_keys_from' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'allow',      'allow_keys',       'warn' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'allow_from', 'allow_keys_from',  'warn' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'follow',     'follow_keys_from', 'warn' );
 }
 
 sub translate_name {
-    my $self              = shift;
-    my $config_class_name = shift;
-    my $elt_name          = shift;
-    my $info              = shift;
-    my $from              = shift;
-    my $to                = shift;
+    my ($self, $config_class_name, $elt_name, $info, $from, $to, $legacy) = @_;
 
     if ( defined $info->{$from} ) {
         $self->show_legacy_issue(
-            "$config_class_name->$elt_name: parameter $from is deprecated in favor of $to");
+            "$config_class_name->$elt_name: parameter $from is deprecated in favor of $to",
+            $legacy
+        );
         $info->{$to} = delete $info->{$from};
     }
 }
@@ -665,18 +718,18 @@ sub translate_compute_info {
             Data::Dumper->Dump( [$compute_info], [qw/compute_info/] )
         ) if $legacy_logger->is_debug;
 
-        $self->show_legacy_issue( "$config_class_name->$elt_name: specifying compute info with ",
-            "an array ref is deprecated" );
+        $self->show_legacy_issue([ "$config_class_name->$elt_name: specifying compute info with ",
+            "an array ref is deprecated" ]);
 
         my ( $user_formula, %var ) = @$compute_info;
         my $replace_h;
         map { $replace_h = delete $var{$_} if ref( $var{$_} ) } keys %var;
 
         # cleanup user formula
-        $user_formula =~ s/\$(\w+){/\$replace{/g;
+        $user_formula =~ s/\$(\w+)\{/\$replace{/g;
 
         # cleanup variable
-        map { s/\$(\w+){/\$replace{/g } values %var;
+        map { s/\$(\w+)\{/\$replace{/g } values %var;
 
         # change the hash *in* the info structure
         $info->{$new_name} = {
@@ -708,10 +761,10 @@ sub translate_id_class {
     my $class_overide = $info->{$class_overide_param};
     if ($class_overide) {
         $info->{class} = $class_overide;
-        $self->show_legacy_issue(
+        $self->show_legacy_issue([
             "$config_class_name->$elt_name: '$class_overide_param' is deprecated, ",
             "Use 'class' instead."
-        );
+        ]);
     }
 
     $legacy_logger->debug(
@@ -738,15 +791,15 @@ sub translate_id_default_info {
     my $def_info = delete $info->{default};
     if ( ref($def_info) eq 'HASH' ) {
         $info->{default_with_init} = $def_info;
-        $self->show_legacy_issue( $warn, "Use default_with_init" );
+        $self->show_legacy_issue([ $warn, "Use default_with_init" ]);
     }
     elsif ( ref($def_info) eq 'ARRAY' ) {
         $info->{default_keys} = $def_info;
-        $self->show_legacy_issue( $warn, "Use default_keys" );
+        $self->show_legacy_issue([ $warn, "Use default_keys" ]);
     }
     else {
         $info->{default_keys} = [$def_info];
-        $self->show_legacy_issue( $warn, "Use default_keys" );
+        $self->show_legacy_issue([ $warn, "Use default_keys" ]);
     }
 
     $legacy_logger->debug( 
@@ -774,11 +827,11 @@ sub translate_id_auto_create {
     if ( $info->{type} eq 'hash' ) {
         $info->{auto_create_keys} =
             ref($ac_info) eq 'ARRAY' ? $ac_info : [$ac_info];
-        $self->show_legacy_issue( $warn, "Use auto_create_keys" );
+        $self->show_legacy_issue([ $warn, "Use auto_create_keys" ]);
     }
     elsif ( $info->{type} eq 'list' ) {
         $info->{auto_create_ids} = $ac_info;
-        $self->show_legacy_issue( $warn, "Use auto_create_ids" );
+        $self->show_legacy_issue([ $warn, "Use auto_create_ids" ]);
     }
     else {
         die "Unexpected element ($elt_name) type $info->{type} ", "for translate_id_auto_create";
@@ -810,6 +863,33 @@ sub translate_id_min_max {
     }
 }
 
+sub translate_warped_node_info {
+    my ( $self, $config_class_name, $elt_name, $type, $info ) = @_;
+
+    $legacy_logger->debug(
+        "translate_warped_node_info $elt_name input:\n",
+        Data::Dumper->Dump( [$info], [qw/info/] )
+    ) if $legacy_logger->is_debug;
+
+    # type may not be defined when translating class snippet used to augment a class
+    my $elt_type = $info->{type} ;
+    foreach my $parm (qw/follow rules/) {
+        next unless $info->{$parm};
+        next if defined $elt_type and $elt_type ne 'warped_node';
+        $self->show_legacy_issue(
+            "$config_class_name->$elt_name: using $parm parameter in "
+            ."warped node is deprecated. $parm must be specified in a warp parameter."
+            ,'note' # TODO later, fall 2016 : issue a warning that may break tests
+        );
+        $info->{warp}{$parm} = delete $info->{$parm};
+    }
+
+    $legacy_logger->debug(
+        "translate_warped_node_info $elt_name output:\n",
+        Data::Dumper->Dump( [$info], [qw/new_info/] )
+    ) if $legacy_logger->is_debug;
+}
+
 # internal: translate warp information into 'boolean expr' => { ... }
 sub translate_warp_info {
     my ( $self, $config_class_name, $elt_name, $type, $warp_info ) = @_;
@@ -983,8 +1063,8 @@ sub translate_legacy_builtin {
         Data::Dumper->Dump( [$normalized_model], ['builtin to translate'] )
     ) if $legacy_logger->is_debug;
 
-    $self->show_legacy_issue( "$config_class_name: parameter 'built_in' is deprecated "
-            . "in favor of 'upstream_default'" );
+    $self->show_legacy_issue([ "$config_class_name: parameter 'built_in' is deprecated "
+            . "in favor of 'upstream_default'" ]);
 
     $model->{upstream_default} = $raw_builtin_default;
 
@@ -1002,8 +1082,8 @@ sub translate_legacy_built_in_list {
         Data::Dumper->Dump( [$normalized_model], ['built_in_list to translate'] )
     ) if $legacy_logger->is_debug;
 
-    $self->show_legacy_issue( "$config_class_name: parameter 'built_in_list' is deprecated "
-            . "in favor of 'upstream_default_list'" );
+    $self->show_legacy_issue([ "$config_class_name: parameter 'built_in_list' is deprecated "
+            . "in favor of 'upstream_default_list'" ]);
 
     $model->{upstream_default_list} = $raw_builtin_default;
 
@@ -1118,7 +1198,7 @@ sub load {
     # look for additional model information
     my %model_graft_by_name;
     my %done;  # avoid loading twice the same snippet (where system version may clobber dev version)
-    foreach my $inc (@INC) {
+    foreach my $inc (@INC,'.') {
         foreach my $name ( keys %models_by_name ) {
             my $snippet_path = $name;
             $snippet_path =~ s/::/\//g;
@@ -1180,6 +1260,7 @@ sub _do_model_file {
     get_logger("Model::Loader")->info("load model $load_file");
 
     my $err_msg = '';
+    $load_file = "./$load_file" if $load_file !~ m!^/! and -e $load_file ;
     my $model   = do $load_file;
 
     unless ($model) {
@@ -1216,7 +1297,9 @@ sub augment_config_class_really {
     croak "unknown class to augment: $config_class_name" unless defined $orig_model;
 
     my $model_addendum = $self->normalize_class_parameters( $config_class_name, $augment_data );
-    my $new_model = merge( $orig_model, $model_addendum );
+
+    my $merge = Hash::Merge->new('RIGHT_PRECEDENT');
+    my $new_model = $merge->merge( $orig_model, $model_addendum );
 
     # remove duplicates in element_list and accept_list while keeping order
     foreach my $list_name (qw/element_list accept_list/) {
@@ -1254,9 +1337,11 @@ sub get_model {
     return dclone($model);
 }
 
+# internal
 sub get_model_doc {
-    my ( $self, $top_class_name ) = @_;
+    my ( $self, $top_class_name, $done ) = @_;
 
+    $done //= {};
     if ( not defined $self->normalized_model($top_class_name) ) {
         croak "get_model_doc error : unknown config class name: $top_class_name";
     }
@@ -1266,7 +1351,8 @@ sub get_model_doc {
 
     while (@classes) {
         my $class_name = shift @classes;
-        next if defined $result{$class_name};
+        next if $done->{$class_name} ;
+
         my $c_model = $self->get_model($class_name)
             || croak "get_model_doc model error : unknown config class name: $class_name";
 
@@ -1280,6 +1366,9 @@ sub get_model_doc {
             "# PODNAME: $full_name",
             "# ABSTRACT:  Configuration class " . $class_name,
 
+            # assume utf8 for all docs
+            "=encoding utf8", '',
+
             # plain old pod compatibility
             "=head1 NAME",                                     '',
             "$full_name - Configuration class " . $class_name, '',
@@ -1345,6 +1434,7 @@ sub get_model_doc {
         );
 
         $result{$full_name} = join( "\n", @pod, @elt, @see_also, @end, '=cut', '' ) . "\n";
+        $done->{$class_name} = 1;
     }
     return \%result;
 }
@@ -1388,7 +1478,8 @@ sub get_element_description {
 
     my $desc = $elt_info->{description} || '';
     if ($desc) {
-        $desc .= '. ' if $desc =~ /\w$/;
+        $desc .= '.' if $desc =~ /\w$/;
+        $desc .= ' ' unless $desc =~ /\s$/;
     }
 
     if ( my $status = $elt_info->{status} ) {
@@ -1426,10 +1517,10 @@ sub get_element_value_help {
 }
 
 sub generate_doc {
-    my ( $self, $top_class_name, $dir ) = @_;
+    my ( $self, $top_class_name, $dir, $done ) = @_;
 
-    my $res = $self->get_model_doc($top_class_name);
-    my @wrote;
+    $done //= {} ;
+    my $res = $self->get_model_doc($top_class_name, $done);
 
     if ( defined $dir and $dir ) {
         foreach my $class_name ( sort keys %$res ) {
@@ -1454,7 +1545,6 @@ sub generate_doc {
                 $fh->print( $res->{$class_name} );
                 $fh->close;
                 print "Wrote documentation in $pod_file\n";
-                push @wrote, $pod_file;
             }
         }
     }
@@ -1464,7 +1554,6 @@ sub generate_doc {
             print $res->{$class_name};
         }
     }
-    return @wrote;
 }
 
 sub get_element_model {
@@ -1597,21 +1686,30 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
-=head2 Perl program
+=head2 Perl program to use an existing model
+
+ use Config::Model qw(cme);
+ # load, modify and save popcon configuration file
+ cme('popcon')->modify("PARTICIPATE=yes");
+
+=head2 Command line to use an existing model
+
+ # with App::Cme
+ cme modify popcon 'PARTICIPATE=yes'
+
+=head2 Perl program with a custom model
 
  use Config::Model;
- use Log::Log4perl qw(:easy) ;
- Log::Log4perl->easy_init($WARN);
 
  # create new Model object
  my $model = Config::Model->new() ; # Config::Model object
 
- # create config model. Most users will want to store the model
- # in lib/Config/Model/models and run cme as explained below
+ # create config model. A more complex model should be stored in a
+ # file in lib/Config/Model/models. Then, run cme as explained below
  $model ->create_config_class (
    name => "MiniModel",
    element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ],
@@ -1634,7 +1732,7 @@ version 2.082
 
  # now look for new mini.ini file un current directory
 
-=head2 More convenient
+=head2 Create a new model file and use it
 
  $ mkdir -p lib/Config/Model/models/
  $ echo "[ { name => 'MiniModel', \
@@ -1644,42 +1742,36 @@ version 2.082
                             } \
            } \
          ] ; " > lib/Config/Model/models/MiniModel.pl
+ # require App::Cme
  $ cme modify -try MiniModel -dev bar=BARV foo=FOOV baz=BAZV
  $ cat mini.ini
 
-=head2 Look Ma, no Perl
-
- $ echo "Make sure that Config::Model::Itself is installed"
- $ mkdir -p lib/Config/Model/models/
- $ config-model-edit -model MiniModel -save \
-   class:MiniModel element:foo type=leaf value_type=uniline - \
-                   element:bar type=leaf value_type=uniline - \
-                   element:baz type=leaf value_type=uniline - \
-   read_config:0 backend=IniFile file=mini.ini config_dir=. auto_create=1 - - -
- $ cme modify -try MiniModel -dev bar=BARV foo=FOOV baz=BAZV
- $ cat mini.ini
+Note that model creation is easier running C<cme meta edit> with
+L<App::Cme> and L<Config::Model::Itself>.
 
 =head1 DESCRIPTION
 
 Config::Model enables a project developer to provide an interactive
 configuration editor (graphical, curses based or plain terminal) to
-his users. For this he must:
+users.
+
+To provide these tools, Config::Model needs:
 
 =over
 
 =item *
 
-Describe the structure and constraints of his project's configuration
-(fear not, a GUI is available)
+A description of the structure and constraints of the project's configuration
+(fear not, a GUI is available with L<App::Cme>)
 
 =item *
 
-Find a way to read and write configuration data using read/write backend
-provided by Config::Model or other Perl modules.
+A module to read and write configuration data. This can be one of the
+read/write backends provided by Config::Model or a custom backend.
 
 =back
 
-With the elements above, Config::Model will generate interactive
+With the elements above, Config::Model generates interactive
 configuration editors (with integrated help and data validation).
 These editors can be graphical (with L<Config::Model::TkUI>), curses
 based (with L<Config::Model::CursesUI>) or based on ReadLine.
@@ -1704,7 +1796,7 @@ single command.
 =head2 How does this work ?
 
 Using this project, a typical configuration editor/validator/upgrader
-will be made of 3 parts :
+is made of 3 parts :
 
   GUI <--------> |---------------|
   CursesUI <---> | |---------|   |
@@ -1718,22 +1810,22 @@ will be made of 3 parts :
 
 =item 1.
 
-A reader and writer that will parse the configuration file and transform
-in a tree representation within Config::Model. The values contained in this
+A reader and writer that parse the configuration file and transform its data
+into a tree representation within Config::Model. The values contained in this
 configuration tree can be written back in the configuration file(s).
 
 =item 2.
 
 A validation engine which is in charge of validating the content and
 structure of configuration stored in the configuration tree. This
-validation engine will follow the structure and constraint declared in
+validation engine follows the structure and constraint declared in
 a configuration model. This model is a kind of schema for the
 configuration tree.
 
 =item 3.
 
 A user interface to modify the content of the configuration tree. A
-modification will be validated instantly by the validation engine.
+modification is validated immediately by the validation engine.
 
 =back
 
@@ -1802,16 +1894,11 @@ Maintenance and evolution of the configuration content is easier
 
 =item *
 
-User will see a *common* interface for *all* programs using this
+User sees a *common* interface for *all* programs using this
 project.
 
 =item *
 
-Beginners will not see advanced parameters (advanced and master
-parameters are hidden from beginners)
-
-=item *
-
 Upgrade of configuration data is easier and sanity check is
 performed during the upgrade.
 
@@ -1851,7 +1938,8 @@ Through a virtual file system where every configuration parameter is mapped to a
 All these interfaces are generated from the configuration model.
 
 And configuration model can be created or modified with a graphical
-user interface (with Config::Model::Itself)
+user interface (with C<cme meta edit> once L<Config::Model::Itself> is
+installed)
 
 =head2 What about configuration data storage ?
 
@@ -1899,6 +1987,12 @@ L<Config::Model::Cookbook::CreateModelFromDoc>
 
 =item *
 
+L<Config::Model::models::Itself::Class>: This doc and its siblings
+describes all parameters available to create a model. These are the
+parameters available in the GUI launched by C<cme meta edit> command.
+
+=item *
+
 L<Config::Model::Manual::ModelCreationAdvanced>
 
 =back
@@ -1915,8 +2009,6 @@ C<Config::Model> class.
 For an introduction to model creation, please check:
 L<Config::Model::Manual::ModelCreationIntroduction>
 
-Dedicated Config::Model::Manual pages will follow soon.
-
 =head1 Storage backend, configuration reader and writer
 
 See L<Config::Model::BackendMgr> for details
@@ -1928,12 +2020,12 @@ of rules. This set of rules is called the configuration model.
 
 =head1 User interface
 
-The user interface will use some parts of the API to set and get
-configuration values. More importantly, a generic user interface will
-need to explore the configuration model to be able to generate at
+The user interface uses some parts of the API to set and get
+configuration values. More importantly, a generic user interface
+needs to analyze the configuration model to be able to generate at
 run-time relevant configuration screens.
 
-Simple text interface if provided in this module. Curses and Tk
+A command line interface is provided in this module. Curses and Tk
 interfaces are provided by L<Config::Model::CursesUI> and
 L<Config::Model::TkUI>.
 
@@ -1943,38 +2035,38 @@ Simply call new without parameters:
 
  my $model = Config::Model -> new ;
 
-This will create an empty shell for your model.
+This creates an object to host your model.
 
 =head1 Configuration Model
 
 To validate a configuration tree, we must create a configuration model
-that will set all the properties of the validation engine you want to
+that defines all the properties of the validation engine you want to
 create.
 
 The configuration model is expressed in a declarative form (i.e. a
-Perl data structure which is always easier to maintain than a lot of
+Perl data structure which should be easier to maintain than a lot of
 code)
 
-Each configuration class contains a set of:
+Each configuration class may contain a set of:
 
 =over
 
 =item *
 
-node element that will refer to another configuration class
+node elements that refer to another configuration class
 
 =item *
 
-value element that will contains actual configuration data
+value elements that contain actual configuration data
 
 =item *
 
-List or hash of node or value elements
+list or hash elements that also contain several node or value elements
 
 =back
 
-By declaring a set of configuration classes and referring them in node
-element, you will shape the structure of your configuration tree.
+The structure of your configuration tree is shaped by the a set of
+configuration classes that are used in node elements,
 
 The structure of the configuration data must be based on a tree
 structure. This structure has several advantages:
@@ -2007,10 +2099,11 @@ relation between nodes and leaves must be added.
 
 =item *
 
-Some configuration part are actually graph instead of a tree (for
-instance, any configuration that will map a service to a
+A configuration may actually be structured as a graph instead as a tree (for
+instance, any configuration that maps a service to a
 resource). The graph relation must be decomposed in a tree with
-special I<reference> relation. See L<Config::Model::Value/Value Reference>
+special I<reference> relations that complete the tree to form a graph.
+See L<Config::Model::Value/Value Reference>
 
 =back
 
@@ -2019,8 +2112,8 @@ with classes. The classes themselves have relations that closely match
 the relation of the object of the configuration tree. But the class
 need not to be declared in a tree structure (always better to reuse
 classes). But they must be declared as a DAG (directed acyclic graph).
-
-=for html <a href="http://en.wikipedia.org/wiki/Directed_acyclic_graph">More on DAGs</a>
+See also
+L<Directed acyclic graph on Wikipedia|http://en.wikipedia.org/wiki/Directed_acyclic_graph">More on DAGs>
 
 Each configuration class declaration specifies:
 
@@ -2044,7 +2137,7 @@ The class elements
 
 =back
 
-Each element will specify:
+Each element specifies:
 
 =over
 
@@ -2111,16 +2204,34 @@ Example:
 
 =head1 Configuration instance
 
-A configuration instance if the staring point of a configuration tree.
-When creating a model instance, you must specify the root class name, I.e. the
-configuration class that is used by the root node of the tree.
+A configuration instance is created from a model and is the starting
+point of a configuration tree.
+
+=head2 instance(...)
+
+An instance must be created with a model name (using the root class
+name) or an application name (as shown by "L<cme> C<list>" command).
+
+For example:
 
  my $model = Config::Model->new() ;
- $model ->create_config_class
-  (
+ $model->instance( application => 'approx');
+
+Or:
+
+ my $model = Config::Model->new() ;
+ # note that the model class is slightly different compared to
+ # application name
+ $model->instance( root_class_name => 'Approx');
+
+A custom configuration class can also be used with C<root_class_name> parameter:
+
+ my $model = Config::Model->new() ;
+ # create_config_class is described below
+ $model ->create_config_class (
    name => "SomeRootClass",
    element => [ ...  ]
-  ) ;
+ ) ;
 
  # instance name is 'default'
  my $inst = $model->instance (root_class_name => 'SomeRootClass');
@@ -2129,14 +2240,28 @@ You can create several separated instances from a model using
 C<name> option:
 
  # instance name is 'default'
- my $inst = $model->instance (root_class_name => 'SomeRootClass',
-                              name            => 'test1');
+ my $inst = $model->instance (
+   root_class_name => 'SomeRootClass',
+   name            => 'test1'
+ );
 
-Usually, model files will be loaded automatically depending on
-C<root_class_name>. But you can choose to specify the file containing
+Usually, model files are loaded automatically using a path matching
+C<root_class_name> (e.g. configuration class C<Foo::Bar> is stored in
+C<Foo/Bar.pl>. You can choose to specify the file containing
 the model with C<model_file> parameter. This is mostly useful for
 tests.
 
+The C<instance> method can also retrieve an instance that has already
+been created:
+
+ my $inst = $model->instance( name => 'test1' );
+
+=head2 cme ( ... )
+
+This method is syntactic sugar for short program. It creates a new
+C<Config::Model> object and returns a new instance. See L</instance(...)>
+for the parameters.
+
 =head1 Configuration class
 
 A configuration class is made of series of elements which are detailed
@@ -2152,32 +2277,32 @@ several other properties:
 Level is C<important>, C<normal> or C<hidden>.
 
 The level is used to set how configuration data is presented to the
-user in browsing mode. C<Important> elements will be shown to the user
-no matter what. C<hidden> elements will be explained with the I<warp>
-notion.
+user in browsing mode. C<Important> elements are shown to the user no
+matter what. C<hidden> elements are well, hidden. Their purpose is
+explained with the I<warp> notion.
 
 =item status
 
 Status is C<obsolete>, C<deprecated> or C<standard> (default).
 
-Using a deprecated element will issue a warning. Using an obsolete
-element will raise an exception.
+Using a deprecated element raises a warning. Using an obsolete
+element raises an exception.
 
 =item description
 
-Description of the element. This description will be used when
+Description of the element. This description is used while
 generating user interfaces.
 
 =item summary
 
-Summary of the element. This description will be used when generating
-user interfaces and may be used in comments when writing the
+Summary of the element. This description is used while generating
+a user interfaces and may be used in comments when writing the
 configuration file.
 
 =item class_description
 
-Description of the configuration class. This description will be used
-when generating user interfaces.
+Description of the configuration class. This description is used
+while generating user interfaces.
 
 =item generated_by
 
@@ -2207,7 +2332,7 @@ elements after a specific element of your including class:
   include_after => "foo" ,
   element => [ bar => ... , foo => ... , baz => ... ]
 
-Now the element of your class will be:
+Now the element of your class are:
 
   ( bar , foo , xyz , baz )
 
@@ -2268,7 +2393,7 @@ You can also load predeclared model.
 
 =head2 load( <model_name> )
 
-This method will open the model directory and execute a C<.pl>
+This method opens the model directory and execute a C<.pl>
 file containing the model declaration,
 
 This perl file must return an array ref to declare models. E.g.:
@@ -2286,15 +2411,15 @@ This perl file must return an array ref to declare models. E.g.:
 
 do not put C<1;> at the end or C<load> will not work
 
-If a model name contain a C<::> (e.g C<Foo::Bar>), C<load> will look for
+When a model name contain a C<::> (e.g C<Foo::Bar>), C<load> looks for
 a file named C<Foo/Bar.pl>.
 
-This method will also look in C<Foo/Bar.d> directory for additional model information.
-Model snippet found there will be loaded with L<augment_config_class>.
+This method also searches in C<Foo/Bar.d> directory for additional model information.
+Model snippet found there are loaded with L<augment_config_class>.
 
 Returns a list containing the names of the loaded classes. For instance, if
 C<Foo/Bar.pl> contains a model for C<Foo::Bar> and C<Foo::Bar2>, C<load>
-will return C<( 'Foo::Bar' , 'Foo::Bar2' )>.
+returns C<( 'Foo::Bar' , 'Foo::Bar2' )>.
 
 =head2 augment_config_class (name => '...', class_data )
 
@@ -2310,17 +2435,13 @@ for more details on creating model plugins.
 Return a hash containing the model declaration (in a deep clone copy of the hash).
 You may modify the hash at leisure.
 
-=head2 get_model_doc
-
-Generate POD document for configuration class.
-
-=head2 generate_doc ( top_class_name , [ directory ] )
+=head2 generate_doc ( top_class_name , directory , [ \%done ] )
 
 Generate POD document for configuration class top_class_name and all
-classes used by top_class_name, and write them on STDOUT or in
-specified directory.
+classes used by top_class_name, and write them in specified directory.
 
-Returns a list of written file names.
+C<\%done> is an optional reference to a hash used to avoid writing
+twice the same documentation when this method is called several times.
 
 =head2 get_element_model( config_class_name , element)
 
@@ -2380,15 +2501,27 @@ Feedback from users are highly desired. If you find this module useful, please
 share your use cases, success stories with the author or with the config-model-
 users mailing list.
 
-=head1 AUTHOR
+=head1 PROJECT FOUNDER
 
-Dominique Dumont, (ddumont at cpan dot org)
+Dominique Dumont, "ddumont@cpan.org"
+
+=head1 CREDITS
+
+In alphabetical order:
+
+  Harley Pig
+
+  Jose Luis Perez Diez
+
+  Krzysztof Tyszecki
+
+  Mathieu Arnold
+
+  Mohammad S Anwar
 
 =head1 LICENSE
 
-    Copyright (c) 2005-2012 Dominique Dumont.
-
-    This file is part of Config-Model.
+    Copyright (c) 2005-2016 Dominique Dumont.
 
     Config-Model is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public License as
@@ -2415,7 +2548,7 @@ L<https://github.com/dod38fr/config-model/wiki/Creating-models>
 
 =head2 Model elements
 
-The arrow shows the inheritance of the classes
+The arrow shows inheritance between classes
 
 =over
 
@@ -2612,7 +2745,7 @@ CPANTS
 
 The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
 
-L<http://cpants.perl.org/dist/overview/Config-Model>
+L<http://cpants.cpanauthors.org/dist/Config-Model>
 
 =item *
 
