@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Annotation;
-$Config::Model::Annotation::VERSION = '2.088';
+$Config::Model::Annotation::VERSION = '2.097';
 use Mouse;
 use English;
 
@@ -134,7 +134,7 @@ sub load {
     my $self = shift;
     my $f    = $self->file;
     return unless -e $f;
-    my $hash = do $f || croak "can't do $f:$!";
+    my $hash = do "./$f" || croak "can't do $f:$!";
     my $root = $self->instance->config_root;
 
     foreach my $path ( keys %$hash ) {
@@ -164,7 +164,7 @@ Config::Model::Annotation - Read and write configuration annotations
 
 =head1 VERSION
 
-version 2.088
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -224,8 +224,12 @@ version 2.088
 =head1 DESCRIPTION
 
 This module provides an object that read and write annotations (a bit
-like comments) to and from a configuration tree and save them in a file (not
-configuration file)
+like comments) to and from a configuration tree and save them in a
+file (not configuration file). This module can be used to save
+annotation for configuration files that do not support comments.
+
+THis module should not be used for configuration files that support
+comments.
 
 Depending on the effective id of the process, the annotation is
 saved in:
@@ -257,6 +261,10 @@ Save annotations in a file (See L<DESCRIPTION>)
 
 Loads annotations from a file (See L<DESCRIPTION>)
 
+=head1 CAVEATS
+
+This module is currently not used.
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
