@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyId;
-$Config::Model::AnyId::VERSION = '2.088';
+$Config::Model::AnyId::VERSION = '2.097';
 use 5.010;
 
 use Mouse;
@@ -20,10 +20,18 @@ use Carp qw/cluck croak carp/;
 use Log::Log4perl qw(get_logger :levels);
 use Storable qw/dclone/;
 use Mouse::Util::TypeConstraints;
+use Scalar::Util qw/weaken/;
 
 extends qw/Config::Model::AnyThing/;
 
+use Mouse::Util::TypeConstraints;
+
+subtype 'KeyArray' => as 'ArrayRef' ;
+coerce 'KeyArray' => from 'Str' => via { [$_] } ;
+
 my $logger = get_logger("Tree::Element::Id");
+my $deep_check_logger = get_logger('DeepCheck');
+my $fix_logger = get_logger("Anything::Fix");
 my $change_logger = get_logger("ChangeTracker");
 
 enum 'DataMode' => [qw/preset layered normal/];
@@ -38,7 +46,52 @@ has data_mode => (
         delete_data_mode => 'delete',
         clear_data_mode  => 'clear',
     },
-    default => sub { {}; } );
+    default => sub { {}; }
+);
+
+# this is cleared and set by set_properties
+has _warpable_check_content_actions => (
+    is      => 'bare', # no direct accessor
+    isa     => 'ArrayRef[CodeRef]',
+    traits  => ['Array'],
+    handles => {
+        add_warpable_check_content   => 'push',
+        clear_warpable_check_content => 'clear',
+        get_all_warpable_content_checks => 'elements',
+    },
+    default => sub { []; }
+);
+
+has _check_content_actions => (
+    is      => 'bare', # no direct accessor
+    isa     => 'ArrayRef[CodeRef]',
+    traits  => ['Array'],
+    handles => {
+        add_check_content   => 'push',
+        get_all_content_checks => 'elements',
+    },
+    default => sub { []; }
+);
+
+# needs_check defaults to 1 to trap bad data right after loading
+has needs_content_check => ( is => 'rw', isa => 'Bool', default => 1 );
+
+has has_fixes => (
+    is => 'ro',
+    isa => 'Num',
+    default => 0,
+    traits => ['Number'],
+    handles => {
+        inc_fixes =>   [ add => 1 ],
+        dec_fixes =>   [ sub => 1 ],
+        add_fixes => 'add',
+    }
+);
+
+# work-around because 'set' does work with Mouse Number trait
+sub flush_fixes {
+    $_[0]->{has_fixes} = 0;
+}
 
 # Some idea for improvement
 
@@ -55,7 +108,12 @@ my @common_hash_params = qw/default_with_init/;
 has \@common_hash_params => ( is => 'ro', isa => 'Maybe[HashRef]' );
 
 my @common_list_params = qw/allow_keys default_keys auto_create_keys/;
-has \@common_list_params => ( is => 'ro', isa => 'Maybe[ArrayRef]' );
+has \@common_list_params => (
+    is => 'ro',
+    isa => 'KeyArray',
+    coerce => 1,
+    default => sub { []; }
+);
 
 my @common_str_params = qw/allow_keys_from allow_keys_matching follow_keys_from
     migrate_keys_from migrate_values_from
@@ -77,7 +135,6 @@ around BUILDARGS => sub {
 has [qw/backup cargo/] => ( is => 'ro', isa => 'HashRef', required => 1 );
 has warp => ( is => 'ro', isa => 'Maybe[HashRef]' );
 has [qw/morph/] => ( is => 'ro', isa => 'Bool' );
-has content_warning_hash => ( is => 'rw', isa => 'HashRef',  default => sub { {}; } );
 has content_warning_list => ( is => 'rw', isa => 'ArrayRef', default => sub { []; } );
 has [qw/cargo_class max_index index_class index_type/] =>
     ( is => 'rw', isa => 'Maybe[Str]' );
@@ -147,6 +204,15 @@ sub set_properties {
 
     $self->set_convert( \%args ) if defined $args{convert};
 
+    $self-> clear_warpable_check_content;
+    map { $self-> add_warpable_check_content($_) } $self-> get_all_content_checks;
+    map {
+        my $method = "check_$_";
+        my $weak_self = $self;
+        weaken($weak_self); # weaken reference loop ($self - check_content - closure - self)
+        $self-> add_check_content( sub { $weak_self->$method(@_);} ) if  $self->{$_};
+    } qw/duplicates/;
+
     Config::Model::Exception::Model->throw(
         object => $self,
         error  => "Undefined index_type"
@@ -335,24 +401,17 @@ sub handle_args {
 
 sub apply_fixes {
     my $self = shift;
-    $logger->debug( $self->location . ": apply_fixes called" );
+    $fix_logger->debug( $self->location . ": apply_fixes called" );
 
-    $self->check_content( fix => 1 );
+    $self->deep_check( fix => 1, logger => $fix_logger );
 
 }
 
-sub has_fixes {
-    my $self = shift;
-    return $self->{nb_of_content_fixes};
-}
-
 my %check_idx_dispatch =
     map { ( $_ => 'check_' . $_ ); }
     qw/follow_keys_from allow_keys allow_keys_from allow_keys_matching
     warn_if_key_match warn_unless_key_match/;
 
-my %check_content_dispatch = map { ( $_ => 'check_' . $_ ); } qw/duplicates/;
-
 my %mode_move = (
     layered => { preset => 1, normal => 1 },
     preset  => { normal => 1 },
@@ -379,40 +438,59 @@ sub notify_change {
 
     return if $self->instance->initial_load and not $args{really};
 
-    $self->needs_check(1);
+    $self-> needs_content_check(1);
     $self->SUPER::notify_change(%args);
 }
 
+# the number of checks is becoming confusing. We have
+# - check_idx to check whether an index is fine. This is called when creating
+#   a new index
+# - check_content: a more expensive check that runs all content checker registered
+#   in this object. By default, none. A plain AnyId can contains a duplicated_content
+#   checker if configured
+# - a deep_check (for lack of a better name): a also expensive check that involve index
+#   versus other part of the config tree. By default, no check is done. This is currently
+#   used only by DPkg model which check if the index value is used elsewhere
+
+# Using plain check in this class is avoided because it's too generic, but a polymorphic
+# entry point is still needed, oh well...
 sub check {
-    my $self = shift;
-    $self->check_content(@_);
+    # since check is not used when creating an index, but called explicitly
+    # so it can be forwarded to deep_check.
+    goto &deep_check; # backward compat
 }
 
-# check globally the list or hash
+sub deep_check {
+    my $self = shift;
+    my @args = @_;
+
+    $deep_check_logger->debug("called on ".$self->name);
+
+    map { $self->check_idx(@args, index => $_); } $self->fetch_all_indexes();
+
+    $self->check_content(@args, logger => $deep_check_logger);
+}
+
+# check globally the list or hash, called by apply_fix or deep_check
 sub check_content {
     my $self = shift;
 
-    my %args = @_ > 1 ? @_ : ( index => $_[0] );
+    my %args = @_ ;
     my $silent    = $args{silent} || 0;
     my $apply_fix = $args{fix}    || 0;
+    my $local_logger = $args{logger} || $logger;
 
-    Config::Model::Exception::Internal->throw(
-        object => $self,
-        error  => "check method: index or key should not be defined"
-    ) if defined $args{index};
-
-    if ( $self->needs_check ) {
+    if ( $self-> needs_content_check ) {
 
+        $local_logger->debug( "Running check_content on ",$self->location );
         # need to keep track to update GUI
-        $self->{nb_of_content_fixes} = 0;    # reset before check
+        $self-> flush_fixes;    # reset before check
 
         my @error;
         my @warn;
 
-        foreach my $key_check_name ( keys %check_content_dispatch ) {
-            next unless $self->{$key_check_name};
-            my $method = $check_content_dispatch{$key_check_name};
-            $self->$method( \@error, \@warn, $apply_fix );
+        foreach my $sub ( $self-> get_all_content_checks ) {
+            $sub->( \@error, \@warn, $apply_fix, $silent );
         }
 
         my $nb = $self->fetch_size;
@@ -424,12 +502,12 @@ sub check_content {
 
         $self->{content_warning_list} = \@warn;
         $self->{content_error_list}   = \@error;
-        $self->needs_check(0);
+        $self-> needs_content_check(0);
 
         return scalar @error ? 0 : 1;
     }
     else {
-        $logger->debug( $self->location, " has not changed, actual check skipped" )
+        $local_logger->debug( $self->location, " has not changed, actual check skipped" )
             if $logger->is_debug;
         my $err = $self->{content_error_list};
         return scalar @$err ? 0 : 1;
@@ -445,7 +523,7 @@ sub check_idx {
     my $idx       = $args{index};
     my $silent    = $args{silent} || 0;
     my $check     = $args{check} || 'yes';
-    my $apply_fix = $check eq 'fix' ? 1 : 0;
+    my $apply_fix = $args{fix} // ($check eq 'fix' ? 1 : 0);
 
     Config::Model::Exception::Internal->throw(
         object => $self,
@@ -487,13 +565,10 @@ sub check_idx {
     }
 
     $self->{idx_error_list} = \@error;
+    $self->{warning_hash}{$idx} = \@warn;
 
-    if (@warn) {
-        $self->{warning_hash}{$idx} = \@warn;
-        map { warn( "Warning in '" . $self->location_short . "': $_\n" ) } @warn unless $silent;
-    }
-    else {
-        delete $self->{warning_hash}{$idx};
+    if (@warn and not $silent and $check ne 'no') {
+        map { warn( "Warning in '" . $self->location_short . "': $_\n" ) } @warn;
     }
 
     return scalar @error ? 0 : 1;
@@ -565,7 +640,7 @@ sub check_warn_unless_key_match {
 }
 
 sub check_duplicates {
-    my ( $self, $error, $warn, $apply_fix ) = @_;
+    my ( $self, $error, $warn, $apply_fix, $silent ) = @_;
 
     my $dup = $self->{duplicates};
     return if $dup eq 'allow';
@@ -599,7 +674,7 @@ sub check_duplicates {
     elsif ( $dup eq 'warn' ) {
         $logger->debug("warning condition: found duplicate @issues");
         push @$warn, "Duplicated value: @issues";
-        $self->{nb_of_content_fixes} += scalar @issues;
+        $self->add_fixes( scalar @issues);
     }
     elsif ( $dup eq 'suppress' ) {
         $logger->debug("suppressing duplicates @issues");
@@ -893,23 +968,20 @@ sub clear_values {
 
 sub warning_msg {
     my ( $self, $idx ) = @_;
-
+    my $list ;
     if ( defined $idx ) {
-        return $self->{warning_hash}{$idx};
+        $list = $self->{warning_hash}{$idx} ;
     }
-    elsif ( scalar %{ $self->{content_warning_hash} } or @{ $self->{content_warning_list} } ) {
-        my @list = @{ $self->{content_warning_list} };
-        push @list,
-            map ( "key $_: " . $self->{content_warning_hash}{$_},
-            keys %{ $self->{content_warning_hash} } );
-        return join( "\n", @list );
+    elsif ( @{ $self->{content_warning_list} } ) {
+        $list = $self->{content_warning_list} ;
     }
+    return $list ? join( "\n", @$list ) : '';
 }
 
 sub has_warning {
     my $self = shift;
 
-    return @{ $self->{content_warning_list} } + keys %{ $self->{content_warning_hash} };
+    return @{ $self->{content_warning_list} };
 }
 
 sub error_msg {
@@ -939,7 +1011,7 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 2.088
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -1101,7 +1173,7 @@ be used with string index type)
 
 When set, the default parameter (or set of parameters) are used as
 default keys hashes and created automatically when the C<keys> or C<exists>
-functions are used on an I<empty> hash..
+functions are used on an I<empty> hash.
 
 You can use C<< default_keys => 'foo' >>, 
 or C<< default_keys => ['foo', 'bar'] >>.
@@ -1268,6 +1340,34 @@ When a warp is applied, the items that do not fit the constraint
 For the max_nb constraint, an exception is raised if a warp
 leads to a number of items greater than the max_nb constraint.
 
+=head1 Content check
+
+By default, this class provides an optional content check that checks
+for duplicated values (when C<duplicates> parameter is set).
+
+Derived classes can register more global checker with the following method.
+
+=head2 add_check_content
+
+This method expects a sub ref with signature C<( $self, $error, $warn,
+$apply_fix )>.  Where C<$error> and C<$warn> are array ref. You can
+push error or warning messages there.  C<$apply_fix> is a
+boolean. When set to 1, the passed method can fix the warning or the
+error. Please make sure to weaken C<$self> to avoid memory cycles.
+
+Example:
+
+ package MyId;
+ use Mouse;
+ extends qw/Config::Model::HashId/;
+ use Scalar::Util qw/weaken/;
+
+ sub setup {
+    my $self = shift;
+    weaken($self);
+    $self-> add_check_content( sub { $self->check_usused_licenses(@_);} )
+}
+
 =head1 Introspection methods
 
 The following methods returns the current value stored in the Id
