@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::IniFile;
-$Config::Model::Backend::IniFile::VERSION = '2.082';
+$Config::Model::Backend::IniFile::VERSION = '2.097';
 use Carp;
 use Mouse;
 use 5.10.0;
@@ -84,7 +84,7 @@ sub read {
         my $comment_path;
 
         # Update section name
-        if ( $vdata =~ /\[(.*)\]/ ) {
+        if ( $vdata =~ /^\s*\[(.*)\]/ ) {
             $section = $force_lc{section} ? lc($1) : $1;
             my $remap = $section_map->{$section} || '';
             if ( $remap eq '!' ) {
@@ -117,7 +117,7 @@ sub read {
             }
         }
         else {
-            my ( $name, $val ) = split( /\s*=\s*/, $vdata );
+            my ( $name, $val ) = split( /\s*=\s*/, $vdata, 2 );
             $name = lc($name) if $force_lc{key};
             $val  = lc($val)  if $force_lc{value};
             $comment_path = $section_path . ' ' . $self->set_or_push( $section_ref, $name, $val );
@@ -142,7 +142,6 @@ sub read {
 
 sub load_data {
     my $self = shift;
-    say "calling load_data on ". ref($self);
     $self->node->load_data(@_);
 }
 
@@ -184,16 +183,20 @@ sub write {
 
     croak "Undefined file handle to write" unless defined $ioh;
 
-    $self->write_global_comment( $ioh, $delimiter );
+    # use the first char of the list as a comment delimeter
+    my $cc = substr($delimiter,0,1);
+    $args{comment_delimiter} = $cc;
+
+    $self->write_global_comment( $ioh, $cc );
 
     # some INI file have a 'General' section mapped in root node
     my $top_class_name = $self->{reverse_section_map}{''};
     if ( defined $top_class_name ) {
         $logger->debug("writing class $top_class_name from reverse_section_map");
-        $self->write_data_and_comments( $ioh, $delimiter, "[$top_class_name]" );
+        $self->write_data_and_comments( $ioh, $cc, "[$top_class_name]" );
     }
 
-    my $res = $self->_write(@_);
+    my $res = $self->_write(%args);
     $ioh->print($res);
 }
 
@@ -405,7 +408,7 @@ Config::Model::Backend::IniFile - Read and write config as a INI file
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -414,11 +417,11 @@ version 2.082
  my $model = Config::Model->new;
  $model->create_config_class (
     name    => "IniClass",
-    element => [ 
+    element => [
         [qw/foo bar/] => {
             type => 'list',
             cargo => {qw/type leaf value_type string/}
-        } 
+        }
     ]
  );
 
@@ -427,18 +430,18 @@ version 2.082
     name => "MyClass",
 
     element => [
-        'ini_class' => {
+        ini_class => {
             type   => 'hash',
-	    index_type => 'string',
-	    cargo => { 
-		type => 'node',
-		config_class_name => 'IniClass' 
-		},
-	    },
+            index_type => 'string',
+            cargo => {
+                type => 'node',
+                config_class_name => 'IniClass'
+            },
+        },
     ],
 
    read_config  => [
-        { 
+        {
             backend => 'IniFile',
             config_dir => '/tmp',
             file  => 'foo.conf',
@@ -456,7 +459,7 @@ version 2.082
 
  $inst->write_back ;
 
-Now C</tmp/foo.conf> will contain:
+Now C</tmp/foo.conf> contains:
 
  ## file written by Config::Model
  [ONE]
@@ -477,22 +480,41 @@ This INI file can have arbitrary comment delimiter. See the example
 in the SYNOPSIS that sets a semi-column as comment delimiter. 
 By default the comment delimiter is '#' like in Shell or Perl.
 
-Note that undefined values are skipped for list element. I.e. if a
-list element contains C<('a',undef,'b')>, the data structure will
-contain C<'a','b'>.
+Note that undefined values are skipped for list element. I.e. when a
+list element contains C<('a',undef,'b')>, the data structure
+contains C<'a','b'>.
 
-=head1 Comments
+=head1 Limitations
 
-This backend tries to read and write comments from configuration file. The
-comments are stored as annotation within the configuration tree. Bear in mind
-that comments extraction is based on best estimation as to which parameter the 
-comment may apply. Wrong estimations are possible.
+=head2 Structure
+
+Structure of the Config::Model must be very simple. Either:
+
+=over
+
+=item *
+
+A single class with hash of leaves elements.
+
+=item *
+
+2 levels of classes. The top level has nodes elements. All other
+classes have only leaf elements.
+
+=back
+
+=head1 Comments in Ini file
+
+This backend tries to read and write comments from configuration
+file. The comments are stored as annotation within the configuration
+tree. Comments extraction is based on best estimation as to which
+parameter the comment may apply. Wrong estimations are possible.
 
 =head1 CONSTRUCTOR
 
 =head2 new ( node => $node_obj, name => 'inifile' ) ;
 
-Inherited from L<Config::Model::Backend::Any>. The constructor will be
+Inherited from L<Config::Model::Backend::Any>. The constructor is
 called by L<Config::Model::BackendMgr>.
 
 =head1 Parameters
@@ -505,6 +527,12 @@ Optional parameters declared in the model:
 
 Change the character that starts comments in the INI file. Default is 'C<#>'.
 
+Some Ini files allows comments to begin with several characters
+(e.g. C<#> or C<;>). In this case, set C<comment_delimiter> to the
+possible characters (e.g "C<#;>"). The first character is used to
+write back comments. (In the example above, comment C<; blah> is
+written back as C<# blah>.
+
 =item store_class_in_hash
 
 See L</"Arbitrary class name">
@@ -573,9 +601,9 @@ The elements of each class can be different. For instance:
   [ B ]
   baz = baz_v
 
-In this case, class C<A> and class C<B> will not use the same configuration class.
+In this case, class C<A> and class C<B> do not use the same configuration class.
 
-The model will have this structure:
+The model has this structure:
 
  Root class 
  |- leaf element foo
@@ -586,7 +614,7 @@ The model will have this structure:
 
 =head2 Arbitrary class name
 
-In this case, the class names can be chosen by the end user. Each class will have the same 
+In this case, the class names can be chosen by the end user. Each class has the same
 elements. For instance:
 
   foo = foo_v
@@ -595,8 +623,8 @@ elements. For instance:
   [ B ]
   bar = bar_v2
 
-In this case, class C<A> and class C<B> will not use the same configuration class.
-The model will have this structure:
+In this case, class C<A> and class C<B> do not use the same configuration class.
+The model has this structure:
 
  Root class 
  |- leaf foo
@@ -654,9 +682,9 @@ Of all parameters passed to this read call-back, only C<io_handle> is
 used. This parameter must be L<IO::File> object already opened for
 read. 
 
-It can also be undef. In this case, C<read()> will return 0.
+It can also be undef. In this case, C<read()> returns 0.
 
-When a file is read,  C<read()> will return 1.
+When a file is read,  C<read()> returns 1.
 
 =head2 write ( io_handle => ... )
 
@@ -664,7 +692,7 @@ Of all parameters passed to this write call-back, only C<io_handle> is
 used. This parameter must be L<IO::File> object already opened for
 write. 
 
-C<write()> will return 1.
+C<write()> returns 1.
 
 =head1 AUTHOR
 
