@@ -9,7 +9,7 @@
 #
 
 package Config::Model::Backend::Yaml;
-$Config::Model::Backend::Yaml::VERSION = '2.086';
+$Config::Model::Backend::Yaml::VERSION = '2.097';
 use Carp;
 use strict;
 use warnings;
@@ -110,7 +110,7 @@ Config::Model::Backend::Yaml - Read and write config as a YAML data structure
 
 =head1 VERSION
 
-version 2.086
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -176,8 +176,8 @@ Note:
 =item *
 
 Undefined values are skipped for list element. I.e. if a
-list element contains C<('a',undef,'b')>, the data structure will
-contain C<'a','b'>.
+list element contains C<('a',undef,'b')>, the data structure
+contains C<'a','b'>.
 
 =item *
 
@@ -189,7 +189,7 @@ written.
 =head2 Class with only one hash element
 
 If the root node contains a single hash or list element, only the
-content of this hash will be written in YAML file.
+content of this hash is written in a YAML file.
 
 For example, if a class contains:
 
@@ -207,7 +207,7 @@ If the configuration is loaded with:
 
  $root->load("baz:one=un baz:two=deux")
 
-Then the written YAML file will B<not> show C<baz>:
+Then the written YAML file does B<not> show C<baz>:
 
  ---
  one: un
@@ -224,7 +224,7 @@ would be written with:
 
 =head2 new ( node => $node_obj, name => 'yaml' ) ;
 
-Inherited from L<Config::Model::Backend::Any>. The constructor will be
+Inherited from L<Config::Model::Backend::Any>. The constructor is
 called by L<Config::Model::BackendMgr>.
 
 =head2 read ( io_handle => ... )
@@ -233,9 +233,9 @@ Of all parameters passed to this read call-back, only C<io_handle> is
 used. This parameter must be L<IO::File> object already opened for
 read. 
 
-It can also be undef. In this case, C<read()> will return 0.
+It can also be undef. In which case C<read()> returns 0.
 
-When a file is read,  C<read()> will return 1.
+When a file is read,  C<read()> returns 1.
 
 =head2 write ( io_handle => ... )
 
@@ -243,7 +243,7 @@ Of all parameters passed to this write call-back, only C<io_handle> is
 used. This parameter must be L<IO::File> object already opened for
 write. 
 
-C<write()> will return 1.
+C<write()> returns 1.
 
 =head1 AUTHOR
 
