@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::BackendMgr;
-$Config::Model::BackendMgr::VERSION = '2.082';
+$Config::Model::BackendMgr::VERSION = '2.097';
 use Mouse;
 
 use Carp;
@@ -606,29 +606,26 @@ sub auto_write_init {
                 my ( $file_ok, $file_path, $fh ) =
                     $self->open_file_to_write( $backend, suffix => $suffix, @wr_args, @_ )
                     unless ( $c->can('skip_open') and $c->skip_open );
-                my $res = eval {
 
-                    # override needed for "save as" button
-                    $backend_obj->$f(
+                # override needed for "save as" button
+                my %backend_args = (
                         @wr_args,
                         io_handle => $fh,
                         file_path => $file_path,
                         object    => $self->node,
                         @_    # override from user
                     );
-                };
+                my $res = eval { $backend_obj->$f( %backend_args ); };
                 $logger->warn( "write backend $backend $c" . '::' . "$f failed: $@" ) if $@;
                 $self->close_file_to_write( $@, $fh, $file_path );
 
-                if (defined $res and $res == 2) {
-                    unlink($file_path);
-                    return 1;
-                }
+                $self->auto_delete($file_path, \%backend_args) if $write->{auto_delete};
+
                 return defined $res ? $res : $@ ? 0 : 1;
             };
         }
 
-        # FIXME: enhance write back mechanism so that different backend *and* different nodse
+        # FIXME: enhance write back mechanism so that different backend *and* different nodes
         # work as expected
         $logger->debug( "registering write $backend in node " . $self->node->name );
         push @{ $self->{write_back} }, [ $backend, $wb ];
@@ -636,6 +633,20 @@ sub auto_write_init {
     }
 }
 
+sub auto_delete {
+    my ($self, $file_path, $args) = @_;
+
+    my $perl_data = $self->node->dump_as_data( full_dump => $args->{full_dump} // 0);
+
+    my $size = ref($perl_data) eq 'HASH'  ? scalar keys %$perl_data
+             : ref($perl_data) eq 'ARRAY' ? scalar @$perl_data
+             :                              $perl_data ;
+    if (not $size) {
+        $logger->info( "Removing $file_path (no data to store)" );
+        unlink($file_path);
+    }
+}
+
 sub write_back {
     my $self = shift;
     my %args = @_;
@@ -719,14 +730,14 @@ sub close_file_to_write {
         $fh->seek( 0, 0 );    # go back to beginning of file
         $fh->print(@$data);
         $fh->close;
-        $error->rethrow if ref($error);
+        $error->rethrow if ref($error) and $error->can('rethrow');
         die $error;
     }
 
     $fh->close;
 
     # check file size and remove empty files
-    unlink($file_path) if -z $file_path;
+    unlink($file_path) if -z $file_path and not -l $file_path;
 }
 
 sub is_auto_write_for_type {
@@ -762,6 +773,7 @@ sub read_perl {
     my %args = @_;
 
     my $file_path = $args{file_path};
+    $file_path = "./$file_path" unless $file_path =~ m!^\.?/!;
     $logger->info("Read Perl data from $file_path");
 
     my $pdata = do $file_path || die "Cannot open $file_path:$!";
@@ -801,11 +813,11 @@ Config::Model::BackendMgr - Load configuration node on demand
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
- # Use BackendMgr to write data in perl data file
+ # Use BackendMgr to write data in Yaml file
  use Config::Model;
 
  # define configuration tree object
@@ -818,7 +830,7 @@ version 2.082
             value_type => 'string'
         },
     ]
- ); 
+ );
 
  $model->create_config_class(
     name => "MyClass",
@@ -826,13 +838,13 @@ version 2.082
     # read_config spec is used by Config::Model::BackendMgr
     read_config => [
         {
-            backend     => 'perl_file',
+            backend     => 'yaml',
             config_dir  => '/tmp/',
-            file        => 'my_class.pl',
+            file        => 'my_class.yml',
             auto_create => 1,
         },
     ],
-    
+
     element => [
         [qw/foo bar/] => {
             type       => 'leaf',
@@ -849,27 +861,28 @@ version 2.082
     ],
  );
 
- my $inst = $model->node->instance( root_class_name => 'MyClass' );
+ my $inst = $model->instance( root_class_name => 'MyClass' );
 
  my $root = $inst->config_root;
 
  # put data
- my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+ my $steps = 'foo=FOO hash_of_nodes:fr foo=bonjour -
    hash_of_nodes:en foo=hello ';
- $root->load( step => $step );
+ $root->load( steps => $steps );
 
  $inst->write_back;
 
- # now look at file /tmp/my_class.pl
+ # now look at file /tmp/my_class.yml
 
 =head1 DESCRIPTION
 
 This class provides a way to specify how to load or store
-configuration data within the model (instead of writing dedicated perl
-code).
+configuration data within the model.
 
-With these specifications, all the configuration information is read
-during creation of a node.
+With these specifications, all configuration information is read
+during creation of a node (which triggers the creation of a backend
+manager object) and written back when L<write_back|/"write_back ( ... )">
+method is called (either on the node or on this backend manager).
 
 =begin comment
 
@@ -882,158 +895,80 @@ Idea: sub-files name could be <instance>%<location>.cds
 
 =end comment
 
-This load/store can be done with different C<backend>:
+This load/store can be done with different backends:
 
 =over
 
-=item cds_file
+=item *
 
-Config dump string (cds) in a file. I.e. a string that describes the
+Any of the C<Config::Model::Backend::*> classes available on your system.
+For instance C<Config::Model::Backend::Yaml>.
+
+=item *
+
+C<cds_file>: Config dump string (cds) in a file. I.e. a string that describes the
 content of a configuration tree is loaded from or saved in a text
-file. See L<Config::Model::Dumper>.
+file. This format is defined by this project. See
+L<Config::Model::Loader/"load string syntax">.
 
-=item IniFile
+=item *
 
-INI files (written with L<Config::Model::Backend::IniFile>. See limitations in 
-L</"Limitations depending on storage">.
-
-=item perl_file
-
-Perl data structure (perl) in a file. See L<Config::Model::DumpAsData>
+C<perl_file>: Perl data structure (perl) in a file. See L<Config::Model::DumpAsData>
 for details on the data structure.
 
-=item custom
+=item * 
 
-Any format when the user provides a dedicated class and function to
-read and load the configuration tree.
+C<custom>: specifies a dedicated class and function to read and load the
+configuration tree. This is provided for backward compatibility and
+should not be used for new projects.
 
 =back
 
-After loading the data, the object registers itself to the
-instance. Then the user can call the C<write_back> method on the
-instance (See L<Config::Model::Instance>) to store all configuration
-information back.
+When needed, C<write_back> method can be called on the instance (See
+L<Config::Model::Instance>) to store back all configuration information.
 
-=head2 Built-in backend
+=head1 Backend specification
 
-C<cds_file>, C<IniFile> and C<perl_file> backend must be specified with
-mandatory C<config_dir> parameter. For instance:
+The backend specification is provided as an attribute of a
+L<Config::Model::Node> specification. These attributes are optional:
+A node without C<read_config> attribute must rely on another node for
+its data to be read and saved.
 
-   read_config  => { backend    => 'cds_file' , 
-                     config_dir => '/etc/cfg_dir',
-                     file       => 'cfg_file.cds', #optional
-                   },
+When needed (usually for the root node), the configuration class is
+declared with a C<read_config> parameter. This parameter is a list
+of possible backend. Usually, only one read backend is needed.
 
-If C<file> is not specified, a file name will be constructed with
-C<< <config_class_name>.<suffix> >> where suffix is C<pl> or C<ini> or C<cds>.
+=head2 Parameters available for all backends
 
-=head2 Plugin backend classes
+The following parameters are accepted by all backends:
 
-A plugin backend class can also be specified with:
-
-  read_config  => [ { backend    => 'foo' , 
-                      config_dir => '/etc/cfg_dir'
-                      file       => 'foo.conf', # optional
-                    }
-                  ]
-
-In this case, this class will try to load C<Config::Model::Backend::Foo>.
-(The class name is constructed with C<ucfirst($backend_name)>)
-
-C<read_config> can also have custom parameters that will passed
-verbatim to C<Config::Model::Backend::Foo> methods:
-
-  read_config  => [ { backend    => 'foo' , 
-                      config_dir => '/etc/cfg_dir',
-                      my_param   => 'my_value',
-                    } 
-                  ]
-
-This C<Config::Model::Backend::Foo> class must inherit
-L<Config::Model::Backend::Any> and is expected to provide the
-following methods:
-
-=over
-
-=item new
-
-Mandatory parameters:
-
- node => ref_to_config_model_node
-
-C<new()> must return the newly created object
-
-=item read
-
-with parameters:
-
- %custom_parameters,      # model data
- root => $root_dir,       # mostly used for tests
- config_dir => $read_dir, # path below root
- file_path => $full_name, # full file name (root+path+file)
- io_handle => $io_file    # IO::File object
- check     => [ yes|no|skip] 
-
-Must return 1 if the read was successful, 0 otherwise.
-
-Following the C<my_param> example above, C<%custom_parameters> will contain 
-C< ( 'my_param' , 'my_value' ) >, so C<read()> will also be called with
-C<root>, C<config_dir>, C<file_path>, C<io_handle> B<and>
-C<<  my_param   => 'my_value' >>.
-
-=item write
-
-with parameters:
-
- %$write,                     # model data
- auto_create => $auto_create, # from model
- backend     => $backend,     # backend name
- config_dir  => $write_dir,   # override from instance
- io_handle   => $fh,          # IO::File object
- write       => 1,            # always
- check       => [ yes|no|skip] ,
- root        => $root_dir,
- backup      => [ undef || '' || suffix ] # backup strategy required by user
-
-Must return 1 if the write was successful, 0 otherwise
-
-If C<io_handle> is defined, the backup has already been done while
-opening the config file. If C<io_handle> is not defined, there's not
-enough information in the model to read the configuration file and
-create the backup. Your write() method will have to do the backup
-requested by user.
-
-=back
-
-=head2 Custom backend
-
-Custom backend is provided to be backward compatible but should not be used
-for new project. Using a plugin backend as described above is preferred.
-
-Custom backend must be specified with a class name that will features
-the methods used to write and read the configuration files:
-
-  read_config  => [ { backend => 'custom' , 
-                      class => 'MyRead',
-                      config_dir => '/etc/foo', # optional
-                      file => 'foo.conf',       # optional
-                    } ]
-
-C<custom> backend parameters are:
-
-=over
-
-=item class
-
-Specify the class that contain the read method
+=over 4
 
 =item config_dir
 
 Specify configuration directory. This parameter is optional as the
 directory can be hardcoded in the custom class. C<config_dir> beginning
-with 'C<~>' will be munged so C<~> is replaced by C<< File::HomeDir->my_data >>.
+with 'C<~>' is munged so C<~> is replaced by C<< File::HomeDir->my_data >>.
 See L<File::HomeDir> for details.
 
+=item file
+
+Specify configuration file name (without the path). This parameter is
+optional as the file name can be hardcoded in the custom class.
+
+The configuration file name can be specified with C<&index> keyword
+when a backend is associated to a node contained in a hash. For instance,
+with C<file> set to C<index.conf>:
+
+ service    # hash element
+   foo      # hash index
+     nodeA  # values of nodeA are stored in foo.conf
+   bar      # hash index
+     nodeB  # values of nodeB are  stored in bar.conf
+
+Alternatively, C<file> can be set to C<->, in which case, the
+configuration is read from STDIN.
+
 =item os_config_dir
 
 Specify alternate location of a configuration directory depending on the OS
@@ -1043,12 +978,6 @@ For instance:
  config_dir => '/etc/ssh',
  os_config_dir => { darwin => '/etc' }
 
-=item file
-
-optional. Configuration file. This parameter may not apply if the
-configuration is stored in several files. By default, the instance name
-is used as configuration file name.
-
 =item default_layer
 
 Optional. Specifies where to find a global configuration file that
@@ -1064,119 +993,121 @@ overridden by user's file:
 
 Only the 3 above parameters can be specified in C<default_layer>.
 
-=item function
-
-Function name that will be called back to read the file. 
-See L</"read callback"> for details. (default is C<read>)
-
 =item auto_create
 
 By default, an exception is thrown if no read was
-successful. This behavior can be overridden by specifying 
+successful. This behavior can be overridden by specifying
 C<< auto_create => 1 >> in one of the backend specification. For instance:
 
-    read_config  => [ { backend => 'cds_file', config_dir => '/etc/my_cfg/' } , 
-                      { backend => 'custom', class => 'Bar' ,
-                        auto_create => 1
-                      },
-                    ],
+    read_config  => [ {
+        backend => 'IniFile',
+        config_dir => '/tmp',
+        file  => 'foo.conf',
+        auto_create => 1
+    } ],
 
-This feature is necessary to create a configuration from scratch
+Setting C<auto_create> to 1 is necessary to create a configuration
+from scratch
 
-When set in write backend, missing directory and files will be created
-with current umask. Default is false.
+When C<auto_create> is set in write backend, missing directory and
+files are created with current umask. Default is false.
+
+=item auto_delete
+
+Delete configuration files that contains no data. (default is to leave an empty file)
 
 =back
 
-Write specification is similar to read_specification. Except that the
-default value for C<function> is C<write>. Here's an example:
+=head2 Config::Model::Backend::* backends
 
-   write_config  => [ { backend => 'cds_file', config_dir => '/etc/my_cfg/' } , 
-                      { backend => 'custom', class => 'Bar' ,
-                        function => 'my_write',
-                      },
-                    ],
+Specify the backend name and the parameters of the backend defined
+in their documentation.
 
-=head1 Limitations depending on storage
+For instance:
 
-Some storage system will limit the structure of the model you can map
-to the file.
+   read_config => [{
+       backend     => 'yaml',
+       config_dir  => '/tmp/',
+       file        => 'my_class.yml',
+   }],
 
-=head2 Ini files limitation
+See L<Config::Model::Backend::Yaml> for more details for this backend.
 
-Structure of the Config::Model must be very simple. Either:
+=head2 Your own backend
+
+You can also write a dedicated backend. See
+L<How to write your own backend|Config::Model::Backend::Any/"How to write your own backend">
+for details.
+
+=head2 Built-in backend
+
+C<cds_file> and C<perl_file> backend must be specified with
+mandatory C<config_dir> parameter. For instance:
+
+   read_config  => { 
+       backend    => 'cds_file' , 
+       config_dir => '/etc/cfg_dir',
+       file       => 'cfg_file.cds', #optional
+   },
+
+When C<file> is not specified, a file name is constructed with
+C<< <config_class_name>.<suffix> >> where suffix is C<pl> or C<cds>.
+
+=head2 Custom backend
+
+Custom backend is provided to be backward compatible but should not be used
+for new project.
+L<Writing your own backend|Config::Model::Backend::Any/"How to write your own backend">
+is preferred.
+
+Custom backend must be specified with a class name that features the
+methods used to write and read the configuration files:
+
+  read_config  => [ {
+      backend    => 'custom' ,
+      class      => 'MyRead',
+      function   => 'read_it",  # optional, defaults to 'read'
+      config_dir => '/etc/foo', # optional
+      file       => 'foo.conf', # optional
+  } ]
+
+C<custom> backend parameters are:
 
 =over
 
-=item *
+=item class
 
-A single class with hash of leaves elements.
+Specify the class that contains the read methods
 
-=item * 
+=item function
 
-2 levels of classes. The top level has nodes elements. All other
-classes have only leaf elements.
+Function name that is called back to read the file.
+See L</"read callback"> for details. (default is C<read>)
+
+=item file
+
+optional. Configuration file. This parameter may not apply if the
+configuration is stored in several files. By default, the instance name
+is used as configuration file name.
 
 =back
 
-=head1 Configuration class with auto read or auto write
+Most of the times, there's no need to create a write specification:
+the read specification is enough for this module to write back the
+configuration file. 
 
-=head2 read specification
+The write method must be specified if the writer class is not the same as the
+reader class or if the writer method is not C<write>:
 
-A configuration class will be declared with optional C<read_config>
-parameter:
+  write_config  => [ {
+      backend    => 'custom' ,
+      class      => 'MyWrite',
+      function   => 'write_it", # optional, defaults to 'read'
+      config_dir => '/etc/foo', # optional
+      file       => 'foo.conf', # optional
+  } ]
 
-  read_config  => [ { backend => 'cds_file', config_dir => '/etc/my_cfg/' } , 
-                    { backend => 'custom', class => 'Bar' },
-                  ],
-
-The read backends will be tried in the specified order:
-
-=over
-
-=item *
-
-First the C<cds> file whose name depend on the parameters used in model
-creation and instance creation:
-C<< <model_config_dir>/<instance_name>.cds >>
-The syntax of the C<cds> file is described in  L<Config::Model::Dumper>.
-
-=item * 
-
-A callback to C<Bar::read>. See L</"read callback> for details.
-
-=back
-
-When a read operation is successful, the remaining read methods will
-be skipped.
-
-=head2 write specification
-
-A configuration class will be declared with optional C<write_config>
-parameters (along with C<read_config> parameter):
-
-  write_config => [ { backend => 'cds_file', config_dir => '/etc/my_cfg/',
-                      auto_create => 1, },
-                    { backend => 'custom', class => 'NewFormat' } ],
-
-By default, the specifications are tried in order, until the first succeeds.
-
-When required by the user, all configuration information is written
-back using B<all> the write specifications. See
-L<Config::Model::Instance/write_back ( ... )> for details.
-
-The write class declared with C<custom> backend must provide a call-back.
-See L</"write callback"> for details.
-
-=head2 read write directory
-
-By default, configurations files are read from the directory specified
-by C<config_dir> parameter specified in the model. You may override the
-C<root> directory for test.
-
-=head2 read callback
-
-Read callback function will be called with these parameters:
+Read callback function is called with these parameters:
 
   object     => $obj,         # Config::Model::Node object 
   root       => './my_test',  # fake root directory, used for tests
@@ -1190,9 +1121,7 @@ The L<IO::File> object is undef if the file cannot be read.
 
 The callback must return 0 on failure and 1 on successful read.
 
-=head2 write callback
-
-Write callback function will be called with these parameters:
+Write callback function is called with these parameters:
 
   object      => $obj,         # Config::Model::Node object 
   root        => './my_test',  # fake root directory, used for tests
@@ -1208,83 +1137,49 @@ The L<IO::File> object is undef if the file cannot be written to.
 
 The callback must return 0 on failure and 1 on successful write.
 
-Configuration file will be deleted if callback returns 2.
+=head1 Using backend to change configuration file syntax
+
+C<read_config> tries all the specified backends. This feature 
+can be used to migrate from one syntax to another.
+
+In this example, backend manager first tries to read an INI file
+and then to read a YAML file:
+
+  read_config  => [ 
+    { backend => 'IniFile', ... },
+    { backend => 'yaml',    ... },
+  ],
+
+When a read operation is successful, the remaining read methods are
+skipped.
+
+Likewise, the C<write_config> specification accepts several backends.
+By default, the specifications are tried in order, until the first succeeds.
+
+In the example above, the migration from INI to YAML can be achieved
+by specifying only the YAML backend:
+
+  write_config => [
+    { backend => 'yaml',    ... },
+  ],
+
+=head1 Test setup
+
+By default, configurations files are read from the directory specified
+by C<config_dir> parameter specified in the model. You may override the
+C<root> directory for test.
 
 =head1 CAVEATS
 
-When both C<config_dir> and C<file> are specified, this class will
-write-open the configuration file (and thus clobber it) before calling
+When both C<config_dir> and C<file> are specified, this class
+write-opens the configuration file (and thus clobber it) before calling
 the C<write> call-back and pass the file handle with C<io_handle>
 parameter. C<write> should use this handle to write data in the target
 configuration file.
 
 If this behavior causes problem (e.g. with augeas backend), the
-solution is either to:
-
-=over
-
-=item *
-
-Set C<file> to undef or an empty string in the C<write_config>
-specification.
-
-=item *
-
-Create a C<skip_open> function in your backend class that returns C<1>
-
-=back
-
-=head1 EXAMPLES
-
-In the example below, only a C<cds> file is written. But, both custom
-format and C<cds> file are tried for read. This is also an example of
-a graceful migration from a customized format to a C<cds> format.
-
-  read_config  => [ { backend => 'cds_file', config_dir => '/etc/my_cfg/' } , 
-                    { backend => 'custom', class => 'Bar' },
-                  ],
-  write_config => [{ backend => 'cds_file', config_dir => '/etc/my_cfg/' }],
-
-You can choose also to read and write only customized files:
-
-  read_config  => [{ backend => 'custom', class => 'Bar'}],
-
-Or to read and write only C<cds> files :
-
-  read_config  => [{ backend => 'cds_file'}] ,
-
-You can also specify more parameters that must be passed to your
-custom class:
-
-  read_config  => [{ backend => 'custom', class => 'Bar', 
-                    config_dir => '/etc/foo'}],
-
-To migrate from an old format to a new format:
-
-  read_config  => [ { backend => 'custom',
-                      class => 'OldFormat',
-                      function => 'old_read'
-                    } ,
-                    { backend => 'custom',
-                      class => 'NewFormat',
-                      function => 'new_read'
-                    }
-                  ],
-  write_config => [ { backend => 'custom',
-                      class => 'NewFormat'
-                    }
-                  ],
-
-If C<write_config> is missing, the data provided by C<read_config>
-will be used. For instance:
-
-  read_config  => [ { backend => 'custom',
-                      class => 'Bar',
-                      config_dir => '/etc/foo'
-                  } ],
-
-In this case, configuration data will be read by C<Bar::read> in
-directory C</etc/foo> and will be written back there by C<Bar::write>.
+solution is either to set C<file> to undef or an empty string in the
+C<write_config> specification.
 
 =head1 Methods
 
@@ -1296,7 +1191,7 @@ true).
 
 You can specify here a pseudo root directory or another config
 directory to write configuration data back with C<root> and
-C<config_dir> parameters. This will override the model specifications.
+C<config_dir> parameters. This overrides the model specifications.
 
 You can force to use a backend by specifying C<< backend => xxx >>.
 For instance, C<< backend => 'perl_file' >> or C<< backend => 'custom' >>.
@@ -1307,7 +1202,7 @@ C<< backend => 'all' >>.
 You can force a specific config file to write with
 C<< config_file => 'foo/bar.conf' >>
 
-C<write_back> will croak if no write call-back are known for this node.
+C<write_back> croaks if no write call-back are known for this node.
 
 =head2 support_annotation
 
