@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::CheckList;
-$Config::Model::CheckList::VERSION = '2.086';
+$Config::Model::CheckList::VERSION = '2.097';
 use Mouse;
 use 5.010;
 
@@ -23,6 +23,8 @@ use Storable qw/dclone/;
 extends qw/Config::Model::AnyThing/;
 
 with "Config::Model::Role::WarpMaster";
+with "Config::Model::Role::Grab";
+with "Config::Model::Role::HelpAsText";
 
 my $logger = get_logger("Tree::Element::CheckList");
 
@@ -744,7 +746,7 @@ Config::Model::CheckList - Handle check list element
 
 =head1 VERSION
 
-version 2.086
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -757,7 +759,7 @@ version 2.086
 
     element => [
 
-        # type check_list will use Config::Model::CheckList
+        # type check_list uses Config::Model::CheckList
         my_check_list => {
             type   => 'check_list',
             choice => [ 'A', 'B', 'C', 'D' ],
@@ -925,13 +927,13 @@ The keys of a hash
 =item *
 
 Another checklist. In this case only the checked items of the other
-checklist will be available.
+checklist are available.
 
 =back
 
 This other hash or other checklist is indicated by the C<refer_to> or
 C<computed_refer_to> parameter. C<refer_to> uses the syntax of the
-C<steps> parameter of L<grab(...)|Config::AnyThing/"grab(...)">
+C<steps> parameter of L<grab(...)|Config::Role::Grab/grab">
 
 See L<refer_to parameter|Config::Model::IdElementReference/"refer_to parameter">.
 
@@ -1059,10 +1061,10 @@ Example:
 
  { A => 0, B => 1, C => 0 , D => 1}
 
-By default, this method will return all items set by the user, or
+By default, this method returns all items set by the user, or
 items set in preset mode or checked by default.
 
-With a parameter, this method will return either:
+With a parameter, this method returns either:
 
 =over
 
@@ -1096,7 +1098,7 @@ model)
 
 =item user
 
-The list set that will be active in the application. (ie. set by user or
+The set that is active in the application. (ie. set by user or
 by layered data or preset or default)
 
 =back
