@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Dumper;
-$Config::Model::Dumper::VERSION = '2.082';
+$Config::Model::Dumper::VERSION = '2.097';
 use Carp;
 use strict;
 use warnings;
@@ -21,11 +21,11 @@ sub new {
 }
 
 sub quote {
-    _quote( qr/(\s|"|\*)/, @_ );
+    _quote( qr/[\s"#*]/, @_ );
 }
 
 sub id_quote {
-    _quote( qr/[\s"\*<>.=#]/, @_ );
+    _quote( qr/[\s"*<>.=#]/, @_ );
 }
 
 sub _quote {
@@ -259,7 +259,7 @@ Config::Model::Dumper - Serialize data of config tree
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -296,8 +296,8 @@ version 2.082
  $root->fetch_element('baz')->fetch_with_id('en')->store('hello') ;
 
  # put more data the easy way
- my $step = 'baz:fr=bonjour baz:hr="dobar dan"';
- $root->load( step => $step ) ;
+ my $steps = 'baz:fr=bonjour baz:hr="dobar dan"';
+ $root->load( steps => $steps ) ;
 
  # dump only customized data
  print $root->dump_tree;
@@ -318,7 +318,7 @@ The serialized string can be used by L<Config::Model::Loader> to store
 the data back into a configuration tree.
 
 Note that undefined values are skipped for list element. I.e. if a list
-element contains C<('a',undef,'b')>, the dump will contain C<'a','b'>.
+element contains C<('a',undef,'b')>, the dump then contains C<'a','b'>.
 
 =head1 CONSTRUCTOR
 
@@ -344,10 +344,10 @@ Parameters are:
 
 =item mode ( full | preset | custom )
 
-C<full> will dump all configuration data including default
+C<full> dumps all configuration data including default
 values.
 
-C<preset> will dump only value entered in preset mode.
+C<preset> dumps only value entered in preset mode.
 
 By default, the dump contains only data modified by the user
 (i.e. C<custom> data that differ from default or preset values).
@@ -362,6 +362,12 @@ nodes and leaves attached to this node are also dumped.
 Skip node that have a write capability matching C<backend_name> in
 their model. See L<Config::Model::BackendMgr>.
 
+This option must be used when using Dumper to write back configuration
+data. When a configuration model contains several backends (one at the
+tree root and others in tree nodes), setting this option ensure that
+the "root" configuration file does not contain data duplicated in
+configuration file of others tree nodes.
+
 =item auto_vivify
 
 Scan and create data for nodes elements even if no actual data was
