@@ -8,18 +8,26 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::HashId;
-$Config::Model::HashId::VERSION = '2.082';
+$Config::Model::HashId::VERSION = '2.097';
 use Mouse;
+use 5.10.1;
 
 use Config::Model::Exception;
 use Carp;
 
+use Mouse::Util::TypeConstraints;
+
+subtype 'HaskKeyArray' => as 'ArrayRef' ;
+coerce 'HaskKeyArray' => from 'Str' => via { [$_] } ;
+
 use Log::Log4perl qw(get_logger :levels);
 
 my $logger = get_logger("Tree::Element::Id::Hash");
 
 extends qw/Config::Model::AnyId/;
 
+with "Config::Model::Role::Grab";
+
 has data => ( is => 'rw', isa => 'HashRef',  default => sub { {}; } );
 has list => (
     is => 'rw',
@@ -31,8 +39,12 @@ has list => (
     }
 );
 
-has [qw/default_keys auto_create_keys/] =>
-    ( is => 'rw', isa => 'ArrayRef', default => sub { []; } );
+has [qw/default_keys auto_create_keys/] => (
+    is => 'rw',
+    isa => 'HaskKeyArray',
+    coerce => 1,
+    default => sub { []; }
+);
 has [qw/morph ordered/] => ( is => 'ro', isa => 'Bool' );
 
 sub BUILD {
@@ -217,6 +229,21 @@ sub sort {
     }
 }
 
+sub insort {
+    my ($self, $id) = @_;
+
+    if ($self->ordered) {
+        my $elt = $self->fetch_with_id($id);
+        $self->_sort;
+        return $elt;
+    }
+    else {
+        Config::Model::Exception::User->throw(
+            object  => $self,
+            message => "cannot call insort on non ordered hash"
+        );
+    }
+}
 
 # hash only method
 sub firstkey {
@@ -276,7 +303,7 @@ sub swap {
 
 sub move {
     my $self = shift;
-    my ( $from, $to ) = @_;
+    my ( $from, $to, %args ) = @_;
 
     Config::Model::Exception::User->throw(
         object  => $self,
@@ -285,7 +312,8 @@ sub move {
 
     my $ok = $self->check_idx($to);
 
-    if ($ok) {
+    my $check = $args{check};
+    if ($ok or $check eq 'no') {
 
         # this may clobber the old content of $self->{data}{$to}
         $self->{data}{$to} = delete $self->{data}{$from};
@@ -323,12 +351,14 @@ sub move {
             $list->[$from_idx] = $to;
         }
     }
-    else {
+    elsif ($check eq 'yes') {
         Config::Model::Exception::WrongValue->throw(
             error  => join( "\n\t", @{ $self->{error} } ),
             object => $self
         );
     }
+    $logger->debug("Skipped move $from -> $to");
+    return $ok;
 }
 
 sub move_after {
@@ -508,7 +538,7 @@ Config::Model::HashId - Handle hash element for configuration model
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -545,6 +575,16 @@ Returns the number of elements of the hash.
 
 Sort an ordered hash. Throws an error if called on a non ordered hash.
 
+=head2 insort
+
+Parameters: key
+
+Create a new element in the ordered hash while keeping alphabetical order of the keys
+
+Returns the newly created element.
+
+Throws an error if called on a non ordered hash.
+
 =head2 firstkey
 
 Returns the first key of the hash. Behaves like C<each> core perl
@@ -563,6 +603,10 @@ Swap the order of the 2 keys. Ignored for non ordered hash.
 
 Rename key1 in key2. 
 
+Also also optional check parameter to disable warning:
+
+ move ('foo','bar', check => 'no')
+
 =head2 move_after ( key_to_move [ , after_this_key ] )
 
 Move the first key after the second one. If the second parameter is
@@ -572,12 +616,12 @@ ordered hash.
 =head2 move_up ( key )
 
 Move the key up in a ordered hash. Attempt to move up the first key of
-an ordered hash will be ignored. Ignored for non ordered hash.
+an ordered hash is ignored. Ignored for non ordered hash.
 
 =head2 move_down ( key )
 
 Move the key down in a ordered hash. Attempt to move up the last key of
-an ordered hash will be ignored. Ignored for non ordered hash.
+an ordered hash is ignored. Ignored for non ordered hash.
 
 =head2 load_data ( data => ( hash_ref | array_ref ) [ , check => ... , ... ])
 
