@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Instance;
-$Config::Model::Instance::VERSION = '2.088';
+$Config::Model::Instance::VERSION = '2.097';
 #use Scalar::Util qw(weaken) ;
 use strict;
 
@@ -22,7 +22,6 @@ use Text::Diff;
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
 
-use Config::Model::Annotation;
 use Config::Model::Exception;
 use Config::Model::Node;
 use Config::Model::Loader;
@@ -234,26 +233,21 @@ has config_file => (is  => 'ro', isa => 'Maybe[Str]');
 
 has config_dir => (is  => 'ro', isa => 'Maybe[Str]');
 
-has skip_read => ( is => 'ro', isa => 'Bool', default => 0 );
-
 has tree => (
     is      => 'ro',
     isa     => 'Config::Model::Node',
     builder => 'reset_config',
     reader  => 'config_root',
-    handles => [qw/apply_fixes/],
+    handles => [qw/apply_fixes deep_check/],
 );
 
 sub reset_config {
     my $self = shift;
 
-    warn "skip_read is deprecated" if $self->skip_read;
-
     return $self->load_node (
         config_class_name => $self->{root_class_name},
         instance          => $self,
         container         => $self,
-        skip_read         => $self->skip_read,
         check             => $self->read_check,
         config_file       => $self->{config_file},
     );
@@ -534,6 +528,7 @@ sub update {
 
     Config::Model::ObjTreeScanner->new(
         node_content_hook => $hook,
+        check => ($args{quiet} ? 'no' : 'yes'),
         leaf_cb => sub { }
     )->scan_node( \@msgs, $root );
 
@@ -558,7 +553,7 @@ Config::Model::Instance - Instance of configuration tree
 
 =head1 VERSION
 
-version 2.088
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -633,11 +628,6 @@ supplied if not provided by the configuration model.
 
 Specify which backend to use. See L</write_back ( ... )> for details
 
-=item skip_read
-
-DEPRECATED. When set, configuration files are not read when
-creating configuration tree.
-
 =item check
 
 Specify whether to check value while reading config files. Either:
@@ -716,6 +706,11 @@ Returns the L<root object|Config::Model::Node> of the configuration tree.
 Scan the tree and apply fixes that are attached to warning specifications. 
 See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>.
 
+=head2 deep_check
+
+Scan the tree and deep check on all elements that support this. Currently only hash or
+list element have this feature.
+
 =head2 needs_save
 
 Returns 1 (or more) if the instance contains data that needs to be
@@ -730,6 +725,12 @@ In scalar context, returns a big string. Useful to print.
 
 Print all changes on STDOUT and return the list of changes.
 
+=head2 clear_changes
+
+Clear list of changes. Note that changes pending in the configuration
+tree is not affected. This clears only the list shown to user. Use
+only for tests.
+
 =head2 has_warning
 
 Returns the number of warning found in the elements of this configuration instance.
