@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Report;
-$Config::Model::Report::VERSION = '2.088';
+$Config::Model::Report::VERSION = '2.097';
 use Carp;
 use strict;
 use warnings;
@@ -44,11 +44,11 @@ sub report {
         if ( defined $value ) {
             my $name = defined $index ? " $element:$index" : $element;
             push @$data_r, $obj->location . " $name = $value";
-            my $desc = $obj->get_help($element);
+            my $desc = $obj->get_help_as_text($element);
             if ( defined $desc and $desc ) {
                 push @$data_r, wrap( "\t", "\t\t", "DESCRIPTION: $desc" );
             }
-            my $effect = $value_obj->get_help($value);
+            my $effect = $value_obj->get_help_as_text($value);
             if ( defined $effect and $effect ) {
                 push @$data_r, wrap( "\t", "\t\t", "SELECTED: $effect" );
             }
@@ -90,7 +90,7 @@ Config::Model::Report - Reports data from config tree
 
 =head1 VERSION
 
-version 2.088
+version 2.097
 
 =head1 SYNOPSIS
 
