@@ -8,11 +8,14 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::SimpleUI;
-$Config::Model::SimpleUI::VERSION = '2.082';
+$Config::Model::SimpleUI::VERSION = '2.097';
 use Carp;
 use 5.010;
 use strict;
 use warnings;
+use open      qw(:std :utf8);    # undeclared streams in UTF-8
+use Encode qw(decode_utf8);
+use Regexp::Common qw/delimited/;
 
 my $syntax = '
 cd <elt> cd <elt:key>, cd - , cd !
@@ -27,15 +30,20 @@ delete elt
    -> like reset, delete a value (set to undef)
 display elt elt:key
    -> display a value
-ls -> show elements of current node
-ll -> show elements of current node and their value
+ls -> show elements of current node (args: filter pattern)
+ll [-nz] [-v] [ element | pattern ]
+   -> show elements of current node and their value
+     (options: -nz → hides empty value, -v → verbose)
+     (args: element name or filter pattern)
 tree -> show configuration tree from current node
 help -> show available command
 desc[ription] -> show class desc of current node
 desc <element>   -> show desc of element from current node
 desc <value> -> show effect of value (for enum)
 changes -> list unsaved changes
-fix -> fix most warnings (called on all elements)
+check [elt] -> run check current on current node or elt
+fix [ ! | elt ]
+  -> fix warnings in current node or of specified element or on all tree (with ! arg)
 save -> save current changes
 exit -> exit shell
 ';
@@ -48,10 +56,14 @@ my $desc_sub = sub {
     if (@_) {
         my $item;
         while ( $item = shift ) {
-            if ( $obj->isa('Config::Model::Node') ) {
+            if ( $obj->get_type() eq 'node' ) {
                 my $type = $obj->element_type($item);
                 my $elt  = $obj->fetch_element($item);
-                $res .= "element $item (type $type): " . $obj->get_help($item) . "\n";
+                my $help = $obj->get_help_as_text($item);
+                $res .= "element $item (type $type)";
+                $res .= ": " if $help;
+                $res .= "\n" if $help =~ /\n/ or length($help) > 40 ;
+                $res .= $help  . "\n" if $help;
                 if ( $type eq 'leaf' and $elt->value_type eq 'enum' ) {
                     $res .= "  possible values: " . join( ', ', $elt->get_choice ) . "\n";
                 }
@@ -59,25 +71,29 @@ my $desc_sub = sub {
         }
     }
     else {
-        $res = $obj->get_help();
+        $res = $obj->get_help_as_text();
     }
     return $res;
 };
 
 my $ll_sub = sub {
     my $self = shift;
-    my $elt  = shift ;
+    my @raw_args = @_;
+
+    my @desc_opt = qw/check no/;
+
+    my %opt = map { /^-(\w+)/; $1 => 1 } grep { /^-/ } @raw_args;
+    push @desc_opt, hide_empty => 1 if $opt{nz} ;
+    push @desc_opt, verbose    => 1 if $opt{v} ;
+
+    my @args = grep {! /^-/ } @raw_args;
+    push @args, '*' unless @args; # default action is to list all elements
 
     my $obj = $self->{current_node};
-    my $res ;
-    if (defined $elt and $elt =~ /\*/) {
-        $elt =~ s/\*/.*/g;
-        $res = $obj->describe( pattern => qr/^$elt$/, check => 'no' );
-    }
-    else {
-        $res = $obj->describe( element => $elt, check => 'no' );
-    }
-    return $res;
+    map {s/\*/.*/g;} @args ;
+    my $pattern = join ('|',@args);
+
+    return $obj->describe( pattern => qr/^$pattern$/, @desc_opt );
 };
 
 my $cd_sub = sub {
@@ -175,10 +191,29 @@ my %run_dispatch = (
         }
         return '';
     },
-    fix => sub {
-        my ( $self ) = @_;
-        return $self->{root}->instance->apply_fixes;
+    check => sub {
+        my ( $self, $elt_name ) = @_;
+        if ($elt_name) {
+            $self->{current_node}->fetch_element($elt_name)->check();
+        }
+        else {
+            $self->{current_node}->check;
+        }
+        return '';
     },
+    fix => sub {
+        my ( $self, $elt_name ) = @_;
+        if ($elt_name eq '!') {
+            $self->{root}->instance->apply_fixes;
+        }
+        elsif ($elt_name) {
+            $self->{current_node}->fetch_element($elt_name)->apply_fixes;
+        }
+        else {
+            $self->{current_node}->apply_fixes;
+        }
+        return '';
+     },
     save => sub {
         my ($self) = @_;
         $self->{root}->instance->write_back();
@@ -257,13 +292,11 @@ sub run {
 
     return '' unless $user_cmd =~ /\w/;
 
-    $user_cmd =~ s/^\s+//;
-
-    my ( $action, $args ) = split( m/\s+/, $user_cmd, 2 );
-    $args =~ s/\s+$//g if defined $args;    #cleanup
+    my $re = $RE{delimited}{-delim=>q{'"}};
+    my ( $action, @args ) = ( $user_cmd =~ /((?:[^\s"']|$re)+)/g );
 
     if ( defined $run_dispatch{$action} ) {
-        my $res = eval { $run_dispatch{$action}->( $self, $args ); };
+        my $res = eval { $run_dispatch{$action}->( $self, @args ); };
         print $@ if $@;
         return $res;
     }
@@ -307,7 +340,7 @@ Config::Model::SimpleUI - Simple interface for Config::Model
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -350,14 +383,15 @@ version 2.082
  my $root = $inst->config_root ;
 
  # put data
- my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+ my $steps = 'foo=FOO hash_of_nodes:fr foo=bonjour -
    hash_of_nodes:en foo=hello ';
- $root->load( step => $step );
+ $root->load( steps => $steps );
 
- my $ui = Config::Model::SimpleUI->new( root => $root ,
-  	               		        title => 'My class ui',
-					prompt => 'class ui',
-				      );
+ my $ui = Config::Model::SimpleUI->new(
+    root => $root ,
+    title => 'My class ui',
+    prompt => 'class ui',
+ );
 
  # engage in user interaction
  $ui -> run_loop ;
@@ -389,7 +423,7 @@ Once the synopsis above has been saved in C<my_test.pl>, you can do:
 
  class ui: hash_of_nodes:en $ ^D
 
-At the end, the test script will dump the configuration tree. The modified
+At the end, the test script dumps the configuration tree. The modified
 C<bar> value can be found in there:
 
  foo=FOO
@@ -438,13 +472,14 @@ Delete a list or hash element
 
 Display a value
 
-=item ls | ls foo*
+=item ls [ pattern ]
 
 Show elements of current node. Can be used with a shell pattern.
 
-=item ll | ll foo*
+=item ll [-nz] [-v] [ pattern ... ]
 
-Describe elements of current node. Can be used with a shell pattern.
+Describe elements of current node. Can be used with shell patterns or element names.
+Skip empty element with C<-nz> option. Display more information with C<-v> option
 
 =item tree
 
@@ -470,10 +505,16 @@ Show effect of value (for enum)
 
 Show unsaved changes
 
+=item check
+
+Without parameter, show warnings starting from current node. With an
+element name as parameter, do the same on the element.
+
 =item fix
 
-Fix most warnings by calling
-L<apply_fixes|Config::Model::Instance/apply_fixes> on instance.
+Try to fix warning starting from current node. With an element name as parameter,
+do the same on the element. With "C<!>" as parameter, try to fix warnings starting
+from root node by calling L<apply_fixes|Config::Model::Instance/apply_fixes> there.
 
 =item exit
 
