@@ -8,10 +8,14 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::TermUI;
-$Config::Model::TermUI::VERSION = '2.082';
+$Config::Model::TermUI::VERSION = '2.097';
 use Carp;
+use utf8;      # so literals and identifiers can be in UTF-8
+use v5.12;     # or later to get "unicode_strings" feature
 use strict;
 use warnings;
+use open      qw(:std :utf8);    # undeclared streams in UTF-8
+use Encode qw(decode_utf8);
 
 use Term::ReadLine;
 
@@ -33,12 +37,26 @@ my $leaf_completion_sub = sub {
     return @ret;
 };
 
-# BUG: When doing autocompletion on a hash element with an index
+my $fix_completion_sub = sub {
+    my ( $self, $text, $line, $start ) = @_;
+
+    my @choice = $self->{current_node}->get_element_name;
+    push @choice, '!';
+    my @ret = grep( /^$text/, @choice );
+    return @ret;
+};
+
+my $ll_completion_sub = sub {
+    my ( $self, $text, $line, $start ) = @_;
+
+    my @choice = $self->{current_node}->get_element_name;
+    push @choice, '-nz';
+    my @ret = grep( /^$text/, @choice );
+    return @ret;
+};
+
+# BUG: autocompletion does not really on a hash element with an index
 # containing white space (i.e. something like std_id:"abc def",
-# readline's completion insists on adding a white space after :
-# i.e. the run command tries 'std_id: "abd def"' . This fails.  The
-# problem probably revolves around setting a readline variable like
-# rl_completer_word_break_characters, but I do not know which one.
 
 my $cd_completion_sub = sub {
     my ( $self, $text, $line, $start ) = @_;
@@ -78,11 +96,11 @@ my $cd_completion_sub = sub {
             foreach my $idx ( $new_item->fetch_element($elt_name)->fetch_all_indexes ) {
                 # my ($idx) = ($raw_idx =~ /([^\n]{1,40})/ );
                 # $idx .= '...' unless $raw_idx eq $idx ;
-                push @choice, "$elt_name:" . ($idx =~ /[^\w._-]/ ? qq("$idx") : $idx ). ' ';
+                push @choice, "$elt_name:" . ($idx =~ /[^\w._-]/ ? qq("$idx") : $idx );
             }
         }
         else {
-            push @choice, "$elt_name ";
+            push @choice, $elt_name;
         }
     }
 
@@ -97,8 +115,10 @@ my $cd_completion_sub = sub {
 my %completion_dispatch = (
     cd     => $cd_completion_sub,
     desc   => $completion_sub,
-    ll     => $completion_sub,
+    ll     => $ll_completion_sub,
     ls     => $completion_sub,
+    check  => $completion_sub,
+    fix    => $fix_completion_sub,
     clear  => $completion_sub,
     set    => $leaf_completion_sub,
     delete => $leaf_completion_sub,
@@ -132,7 +152,7 @@ sub new {
 
     foreach my $p (qw/root title prompt/) {
         $self->{$p} = delete $args{$p}
-            or croak "WizardHelper->new: Missing $p parameter";
+            or croak "TermUI->new: Missing $p parameter";
     }
 
     $self->{current_node} = $self->{root};
@@ -149,6 +169,8 @@ sub new {
         my $attribs = $term->Attribs;
         $attribs->{completion_function}             = $sub_ref;
         $attribs->{completer_word_break_characters} = $word_break_string;
+        # this method is available only on Term::ReadLine::Gnu > 1.32
+        $term->enableUTF8 if $term->can('enableUTF8');
     }
     elsif ( $term->ReadLine eq "Term::ReadLine::Perl" ) {
         no warnings "once";
@@ -176,7 +198,7 @@ sub run_loop {
     my $user_cmd;
     while ( defined( $user_cmd = $term->readline( $self->prompt ) ) ) {
         last if $user_cmd eq 'exit' or $user_cmd eq 'quit';
-
+        $user_cmd = decode_utf8($user_cmd,1);
         #print $OUT "cmd: $user_cmd\n";
         my $res = $self->run($user_cmd);
         print $OUT $res, "\n" if defined $res and $res;
@@ -197,7 +219,7 @@ sub run_loop {
 
 1;
 
-# ABSTRACT: Provides Config::Model UI with Term::ReadLine
+# ABSTRACT: Interactive command line interface for cme
 
 __END__
 
@@ -207,11 +229,11 @@ __END__
 
 =head1 NAME
 
-Config::Model::TermUI - Provides Config::Model UI with Term::ReadLine
+Config::Model::TermUI - Interactive command line interface for cme
 
 =head1 VERSION
 
-version 2.082
+version 2.097
 
 =head1 SYNOPSIS
 
@@ -254,14 +276,15 @@ version 2.082
  my $root = $inst->config_root ;
 
  # put data
- my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+ my $steps = 'foo=FOO hash_of_nodes:fr foo=bonjour -
    hash_of_nodes:en foo=hello ';
- $root->load( step => $step );
+ $root->load( steps => $steps );
 
- my $ui = Config::Model::TermUI->new( root => $root ,
-  	               	              title => 'My class ui',
-			              prompt => 'class ui',
-				      );
+ my $ui = Config::Model::TermUI->new(
+     root => $root ,
+     title => 'My class ui',
+     prompt => 'class ui',
+ );
 
  # engage in user interaction
  $ui -> run_loop ;
@@ -292,7 +315,7 @@ TAB completion:
 
  class ui: hash_of_nodes:en $ ^D
 
-At the end, the test script will dump the configuration tree. The modified
+At the end, the test script dumps the configuration tree. The modified
 C<bar> value can be found in there:
 
  foo=FOO
@@ -309,8 +332,15 @@ on top of L<Term::ReadLine>. To get better interaction you must
 install either L<Term::ReadLine::Gnu> or L<Term::ReadLine::Perl>.
 
 Depending on your installation, either L<Term::ReadLine::Gnu> or
-L<Term::ReadLine::Perl>. See L<Term::ReadLine> to override default
-choice.
+L<Term::ReadLine::Perl> is used. See L<Term::ReadLine> to
+override default choice.
+
+=head1 Dependencies
+
+This module is optional and depends on L<Term::ReadLine> to work. To
+reduce the dependency list of L<Config::Model>, C<Term::ReadLine> is
+only recommended. L<cme> gracefully degrades to
+L<Config::Model::SimpleUI> when necessary.
 
 =head1 USER COMMAND SYNTAX
 
