@@ -78,12 +78,15 @@
             },
             warp2 => {
                 type              => 'warped_node',
-                follow            => '! tree_macro',
                 config_class_name => 'SubSlave',
                 morph             => 1,
-                rules             => [
-                    mXY => { config_class_name => 'SubSlave2' },
-                    XZ  => { config_class_name => 'SubSlave2' } ]
+                warp => {
+                    follow            => '! tree_macro',
+                    rules             => [
+                        mXY => { config_class_name => 'SubSlave2' },
+                        XZ  => { config_class_name => 'SubSlave2' }
+                    ]
+                }
             },
             Y => {
                 type       => 'leaf',
@@ -144,14 +147,16 @@
             },
             warp => {
                 type              => 'warped_node',
-                follow            => '! tree_macro',
                 config_class_name => 'SlaveY',
                 morph             => 1,
-                rules             => [
-
-                    #XY => { config_class_name => 'SlaveY'},
-                    mXY => { config_class_name => 'SlaveY' },
-                    XZ  => { config_class_name => 'SlaveZ' } ]
+                warp => {
+                    follow            => '! tree_macro',
+                    rules             => [
+                        #XY => { config_class_name => 'SlaveY'},
+                        mXY => { config_class_name => 'SlaveY' },
+                        XZ  => { config_class_name => 'SlaveZ' }
+                    ]
+                }
             },
 
             'slave_y' => {
